<?php

namespace HCM\FPA\Entity;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\TkEntity;

class FpaManadmagnetEntity extends TkEntity
{
    private $objManadMagnet;
    private $nrmanadmagnet;
    private $nrorg;
    private $mescompetencia;
    private $cdinscestrut;
    private $dtadmissaovinc;
    private $idtiporegistro;
    private $dslinharegistro;
    private $nrvinculom;
    private $nrpessoarespons;
    private $idativo;
    private $dtinclusao;
    private $dtultatu;
    private $nrorginclusao;
    private $cdoperinclusao;
    private $nrorgultatu;
    private $cdoperultatu;
    private $nmpessoa;
    private $nrcpfpessoa;
    private $nrestrutlegal;

    public function __construct($entityManager, $hcmParameters, $nrOrg, $mescompetencia, $cdInscEstrut){
        parent::__construct(null, $entityManager);
        $this->entityManager  = $entityManager;
        $this->hcmParameters  = $hcmParameters;
        $this->manadMagnet    = Repositories::FPA_MANADMAGNET;
        $this->nrorg          = $nrOrg;
        $this->mescompetencia = $mescompetencia;
        $this->cdinscestrut   = $cdInscEstrut;
        $this->dtinclusao     = DateUtil::getDataAtual();
        $this->idativo        = 'S';
    }
    
    private function deParaCodReceita($CodReceita){
        switch($CodReceita){
            case '0561':
                $this->dsnaturezarend = 'Rendimentos do trabalho assalariado';
                break;
            case '0588':
                $this->dsnaturezarend = 'Trabalho sem Vínculo Empregatício';
                break;
            case '3562':
                $this->dsnaturezarend = 'Participação nos Lucros';
                break;
        }
    }

    public function retornaRegistros(){
        try{
            return $this->entityManager->getRepository(Repositories::FPA_MANADMAGNET)->findBy([
                    'nrorg'          => $this->nrorg,
                    'mescompetencia' => DateUtil::getDataDeString($this->mescompetencia, DateUtil::FORMATO_BRASILEIRO, true),
                    'cdinscestrut'   => $this->cdinscestrut]);
        } catch(\Exception $e) {
            throw new \Exception('Não foi possivel localizar registros do MANAD no magnético. O erro foi: '.$e->getMessage());
        }
    }

    public function qdeRegistros(){
        $cont = 0;
        try{
            $registros = $this->retornaRegistros();
            $cont = count($registros);
        } catch(\Exception $e) {
            throw new \Exception('Não foi possivel localizar registros do MANAD no magnético. O erro foi: '.$e->getMessage());
        }
        return $cont;
    }

    public function addRegCabecal($cdInscEstrut, $nrPessoaRespons, $idAtivo){
        //var_dump('aqui addRegCabecal');
        //var_dump('$nrPessoaRespons', $nrPessoaRespons);

        $this->cdinscestrut    = $cdInscEstrut;
        $this->nrpessoarespons = $nrPessoaRespons;
        $this->idativo         = $idAtivo;
        $this->dtinclusao      = DateUtil::getDataAtual();
        $this->cdoperinclusao  = $this->hcmParameters->getCdoperador();
        $this->cdoperultatu    = $this->hcmParameters->getCdoperador();
        $this->dtultatu        = DateUtil::getDataAtual();
        $this->nrorginclusao   = $this->nrorg;
        $this->nrorgultatu     = $this->nrorg;
    }
    
    public function addRegVinculo($nrCpf, $nmpessoa, $dtAdmissao, $idTipoRegistro, $registro, $nrestrutlegal = null, $nrvinculom){   
        $this->nrvinculom     = $nrvinculom;
        $this->nrestrutlegal  = $nrestrutlegal;
        $this->nrcpfpessoa    = $nrCpf;
        $this->nmpessoa       = $nmpessoa;
        $this->dtadmissaovinc = $dtAdmissao;
        $this->idtiporegistro = $idTipoRegistro;
        $this->idrec = $idrec;
        $this->bpfdec = $bpfdec;
        $this->dslinharegistro .= is_null($this->dslinharegistro) ? $registro : "\n".$registro;
        //$this->deParaCodReceita($CodReceita);
    }

    public function fimAddRegVinculo(){
        $this->setAllToObj();
        $this->persistir();     
        $this->zeraRegVinculo();    
    }

    public function excluirCompetencia(){
        $cont = 0;
        try{
            $cont = $this->qdeRegistros();
            
            $query = $this->entityManager->getConnection()->prepare("
                DELETE FROM FPA_MANADMAGNET
                 WHERE NRORG          = :P_NRORG
                   AND MESCOMPETENCIA = :P_MESCOMPETENCIA
                   AND CDINSCESTRUT   = :P_CDINSCESTRUT");
    
            $query->execute([
                "P_NRORG"          => $this->nrorg,
                "P_MESCOMPETENCIA" => $this->mescompetencia->format('d/m/Y'),
                "P_CDINSCESTRUT"   => $this->cdinscestrut]);
            
        } catch(\Exception $e) {
            throw new \Exception('Não foi possivel retroceder o MANAD no magnético. O erro foi: '.$e->getMessage());
        }
        return $cont;
    }

    private function setAllToObj(){ 
        //var_dump('aqui setAllToObj');

        $this->objManadMagnet = new $this->manadMagnet;
        $this->nrmanadmagnet = \HCM\Util\NovoCodigo::geraCodigo($this->nrorg,'FPA_MANADMAGNET'.$this->nrorg, 1, 22, true ); 
        $this->objManadMagnet->setNrmanadmagnet($this->nrmanadmagnet);
        $this->objManadMagnet->setNrorg($this->nrorg);
        $this->objManadMagnet->setMescompetencia(DateUtil::getDataDeString($this->mescompetencia, DateUtil::FORMATO_BRASILEIRO, true));
        $this->objManadMagnet->setCdinscestrut($this->cdinscestrut);
        $this->objManadMagnet->setDtadmissaovinc(DateUtil::getDataDeString($this->dtadmissaovinc, DateUtil::FORMATO_BRASILEIRO, true));
        $this->objManadMagnet->setIdtiporegistro($this->idtiporegistro);
        $this->objManadMagnet->setDslinharegistro($this->dslinharegistro);
        $this->objManadMagnet->setNrvinculom($this->nrvinculom);
        $this->objManadMagnet->setNrpessoarespons($this->nrpessoarespons);
        $this->objManadMagnet->setIdativo($this->idativo);
        $this->objManadMagnet->setDtinclusao(DateUtil::getDataAtual());
        $this->objManadMagnet->setDtultatu(DateUtil::getDataAtual());
        $this->objManadMagnet->setNrorginclusao($this->nrorg);
        $this->objManadMagnet->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $this->objManadMagnet->setNrorgultatu($this->nrorg);
        $this->objManadMagnet->setCdoperultatu($this->hcmParameters->getCdoperador());
        $this->objManadMagnet->setNmpessoa($this->nmpessoa);
        $this->objManadMagnet->setNrcpfpessoa($this->nrcpfpessoa);
        $this->objManadMagnet->setNrestrutlegal($this->nrestrutlegal);
    }
        
    private function zeraRegCabecal(){
        $this->nrestrutlegal   = null;
        $this->cdinscestrut    = null;
        $this->idtiporegistro  = null;
        $this->nrpessoarespons = null;
        $this->idativo         = null;
    }

    private function zeraRegVinculo(){
        $this->dslinharegistro = null;
        $this->nrvinculom      = null;
        $this->dtadmissaovinc  = null;
        $this->nmpessoa        = null;
        $this->nrcpfpessoa     = null;
        $this->objManadMagnet  = null;
    }
    
    private function persistir(){
        try{
            $this->entityManager->persist($this->objManadMagnet);
            $this->entityManager->flush();
        } catch(\Exception $e) {
            throw new \Exception('Não foi possivel salvar o MANAD no magnético. O erro foi: '.$e->getMessage());
        }
    }
}