<?php

namespace HCM\FPA\ObrigacaoLegal\Manad;

use HCM\FPA\ObrigacaoLegal\Manad\Blocos\Registros\Registro0000;
use HCM\FPA\ObrigacaoLegal\Manad\Blocos\BlocoK;
use HCM\FPA\ObrigacaoLegal\Manad\QueryManad;

class Manad {

    private $hcmParameters;
    private $_objMagnetico;
    private $_estrutResp;
    private $_dsOrdem;
    private $_dsInicio;
    private $_dsFim;
    private $_dtPeriodIni;
    private $_dtPeriodFim;
    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_dtCompetencia;
    private $_blocoK;
    private $queryManad;
    private $_nrpessoa;

    const ql_msg = '<br>'; //Quebra de linha para as mensagens

    public function __construct($entityManager, $hcmParameters, $objMagnetico, $estrutResp, $dsOrdem, $dsInicio, $dsFim, $dtPeriodIni, $dtPeriodFim) {
        //var_dump('aqui __construct class Manad');

        $this->entityManager     = $entityManager;
        $this->hcmParameters     = $hcmParameters;
        $this->_objMagnetico     = $objMagnetico;
        $this->_estrutResp       = $estrutResp;
        $this->_dsOrdem          = $dsOrdem;
        $this->_dsInicio         = $dsInicio;
        $this->_dsFim            = $dsFim;
        $this->_dtPeriodIni      = $dtPeriodIni;
        $this->_dtPeriodFim      = $dtPeriodFim;
        $this->_nrOrg            = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao      = $hcmParameters->getNrorgPadrao();
        $this->_dtCompetencia    = $hcmParameters->getDtCompetencia();

        //var_dump('$this->_estrutResp', $this->_estrutResp);

        $this->_blocoK    = new BlocoK();
        $this->queryManad = new QueryManad($this->entityManager,$this->hcmParameters);
    }

    public function geraManad($nrpessoa) {
        var_dump('aqui geraManad class Manad');
        
        $this->_nrpessoa = $this->getDataPessoa($nrpessoa);
        //var_dump('$this->_nrpessoa', $this->_nrpessoa); //die;

        $this->_preparaBlocoK();
    }

    private function getDataPessoa($nrpessoa) {
        try{
            $params = ['P_NRORG'            => $this->_nrOrg,
                       'P_DTMESCOMPETENC'   => $this->_dtCompetencia,
                       'P_NRPESSOA'         => $nrpessoa];

            return $this->queryManad->get_dataPessoa($params);
        } catch(\Exception $e){
            throw new \Exception('Erro ao buscar dados da nrpessoa: '.$nrpessoa.'. '.self::ql_msg.'Error:'.$e->getMessage());
        }
    }

    private function _preparaBlocoK(){
        var_dump('aqui _preparaBlocoK');

        $this->_preparaRegistroK050();
        $this->_objMagnetico->fimAddRegVinculo();

        $this->_preparaRegistroK100();
        $this->_objMagnetico->fimAddRegVinculo();
    }

    public function _geraRegistroK001(){ //Abertura bloco k
        $this->_blocoK->_registroK001->geraRegistroK001(); 
    }

    private function _preparaRegistroK050(){
        var_dump('aqui _preparaRegistroK050');
        
        try{
            $params = ['P_NRORG'            => $this->_nrOrg,
                       'P_DTMESCOMPETENC'   => $this->_dtCompetencia,
                       'P_NRPESSOA'         => $this->_nrpessoa['NRPESSOA']];

            $sqlResult = $this->queryManad->get_regK050($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco K, Registro tipo K050 , item 4.4.'.self::ql_msg.'Error:'.$e->getMessage());
        }
        //var_dump('sqlResult', $sqlResult); die;

        if(!empty($sqlResult)) {
            $cnpjcei = !empty($sqlResult['CDCNPJEST']) ? $sqlResult['CDCNPJEST'] : $sqlResult['CDCEIEST'];
            $dtIncAlt = $sqlResult['DTADMISSAOVINC'];
            $regTrab = $sqlResult['NRVINCULOM'];
            $cpf  = $sqlResult['NRCPFPESSOA'];
            $nit  = $sqlResult['NRPISPASEPPES'];
            $categ = $sqlResult['CODCATEGTRAB'];
            $nome   = $sqlResult['NMVINCULOM'];
            $dtNasc = $sqlResult['DTNASCPESSOA'];
            $dtAdm  = $sqlResult['DTADMISSAOVINC'];
            $dtDem  = $sqlResult['DTRESCISAOVINC'];
            $indVinc = $this->deParaIndTpVinculo($sqlResult['NRVINCULOEMPREG']);
                
            $this->_blocoK->_registroK050->setCnpjCei($cnpjcei);
            $this->_blocoK->_registroK050->setDtIncAlt($dtIncAlt);
            $this->_blocoK->_registroK050->setRegTrab($regTrab);
            $this->_blocoK->_registroK050->setCpf($cpf);
            $this->_blocoK->_registroK050->setNit($nit);
            $this->_blocoK->_registroK050->setCateg($categ);
            $this->_blocoK->_registroK050->setNome($nome);
            $this->_blocoK->_registroK050->setDtNasc($dtNasc);
            $this->_blocoK->_registroK050->setDtAdm($dtAdm);
            $this->_blocoK->_registroK050->setDtDem($dtDem);
            $this->_blocoK->_registroK050->setIndVinc($indVinc);
            $this->_blocoK->_registroK050->setTpAtoNom(null);
            $this->_blocoK->_registroK050->setNrAtoNom(null);
            $this->_blocoK->_registroK050->setdtAtoNom(null);
        }
        
        $this->_blocoK->_registroK050->geraRegistroK050();
        
        $registro = $this->_blocoK->_registroK050->getLinhaRegistro();
        $this->_addRegVinculo(Registro0000::REGISTRO_K050, $registro);

        $this->_blocoK->_registroK050->setLinhaRegistro();
    }

    private function _addRegVinculo($tpRegistro, $linhaRegistro){
        $this->_objMagnetico->addRegVinculo(
            $this->_nrpessoa['NRCPFPESSOA'], $this->_nrpessoa['NMVINCULOM'], $this->_nrpessoa['DTADMISSAOVINC'], $tpRegistro, $linhaRegistro, $this->_nrpessoa['NRESTRUTLEGAL'],
            $this->_nrpessoa['NRVINCULOM']
        );
    }

    private function deParaIndTpVinculo($nrvinculoempreg){
        $value = null;

        switch($nrvinculoempreg){
            case '1':  //Trabalhador urbano vinculado a empregador pessoa jurídica por contrato de trabalho regido pela CLT, por prazo indeterminado
            case '2':  //Trabalhador urbano vinculado a empregador pessoa física por contrato de trabalho regido pela CLT, por prazo indeterminado
            case '3':  //Trabalhador rural vinculado a empregador pessoa jurídica por contrato de trabalho regido pela Lei nº 5.889/1973, por prazo indeterminado
            case '4':  //Trabalhador rural vinculado a empregador pessoa física por contrato de trabalho regido pela Lei nº 5.889/1973, por prazo indeterminado
            case '10': //Trabalhador urbano vinculado a empregador pessoa jurídica por contrato de trabalho regido pela CLT, por tempo determinado ou obra certa
            case '11': //Trabalhador urbano vinculado a empregador pessoa física por contrato de trabalho regido pela CLT, por tempo determinado ou obra certa
            case '12': //Trabalhador rural vinculado a empregador pessoa jurídica por contrato de trabalho regido pela Lei nº 5.889/1973, por prazo determinado
            case '13': //Trabalhador rural vinculado a empregador pessoa física por contrato de trabalho regido pela Lei nº 5.889/1973, por prazo determinado
                $value = 1; //vínculo normal
                break;
            case '8':  //Trabalhador temporário, regido pela Lei no 6.019, de 3 de janeiro de 1974
                $value = 2; //vínculo temporário com redução de contribuições/obrigação
                break;
            case '5':  //Servidor regido pelo Regime Jurídico Único (federal, estadual e municipal) e militar, vinculado a Regime Próprio de Previdência
                $value = 3; //Servidor público Efetivo Estatutário
                break;
            case '22': //Servidor regido pelo Regime Jurídico Único (federal, estadual e municipal) e militar, vinculado ao Regime Geral de Previdência Social
                $value = 4; //Servidor público Efetivo CLT
                break;
            case '6':  //Servidor público não efetivo (demissível ad nutum ou admitido por meio de legislação especial, não-regido pela CLT)
                $value = 5; //Servidor público Comissionado
                break;
            case '14': //Contribuinte individual - Diretor não empregado e demais empresários sem FGTS
            case '21': //Contribuinte individual – Trabalhador autônomo ou a este equiparado, inclusive o operador de máquina, com contribuição sobre remuneração; trabalhador associado à cooperativa de produção
            case '24': //Contribuinte Individual - Transportador autônomo, com contribuição sobre remuneração.
            case '26': //Contribuinte individual - Diretor não empregado com FGTS (Lei nº 8.036/90, art.16);
            case '27': //Contribuinte individual – Microempreendedor individual
            case '31': //Contribuinte individual - Empresário, sócio e membro de conselho de administração ou fiscal
                $value = 8; //Contribuinte Individual
                break;
            default:
                $value = 9; //Outros vínculos
                break;
        }
        return $value;
    }

    private function _preparaRegistroK100(){
        var_dump('aqui _preparaRegistroK100');
        
        try{
            $params = ['P_NRORG'              => $this->_nrOrg,
                       'P_DTMESCOMPETENC'     => $this->_dtCompetencia,
                       'P_NRVINCULOM'         => $this->_nrpessoa['NRVINCULOM'],
                       'P_NRTIPOESTRUTGEREN'  => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
                       'P_NRTIPOESTRUTOMADOR' => $this->hcmParameters->getTipoEstruturaTomadorOrg()];

            $sqlResult = $this->queryManad->get_regK100($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco K, Registro tipo K100 , item 4.4.'.self::ql_msg.'Error:'.$e->getMessage());
        }
        //var_dump('sqlResult', $sqlResult); //die;

        if(!empty($sqlResult)) {
            $dtIncAlt = $sqlResult['DTINIVIGENCIA'];
            $codLtc  = $sqlResult['NRESTRUTURAM'];
            $cnpjcei = !empty($sqlResult['CDCNPJEST']) ? $sqlResult['CDCNPJEST'] : $sqlResult['CDCEIEST'];
            $descLtc = $sqlResult['NMRAZSOCESTRUT'];
            $cnpjCeiTom  = !empty($sqlResult['CDCNPJTOM']) ? $sqlResult['CDCNPJTOM'] : $sqlResult['CDCEITOM'];
                
            $this->_blocoK->_registroK100->setDtIncAlt($dtIncAlt);
            $this->_blocoK->_registroK100->setCodLtc($codLtc);
            $this->_blocoK->_registroK100->setCnpjCei($cnpjcei);
            $this->_blocoK->_registroK100->setDescLtc($descLtc);
            $this->_blocoK->_registroK100->setCnpjCeiTom($cnpjCeiTom);
        }
        
        $this->_blocoK->_registroK100->geraRegistroK100();
        
        $registro = $this->_blocoK->_registroK100->getLinhaRegistro();
        $this->_addRegVinculo(Registro0000::REGISTRO_K100, $registro);

        $this->_blocoK->_registroK100->setLinhaRegistro();
    }

    public function _preparaRegistroK150(){ //registro geral fora de vínculo.
        var_dump('aqui _preparaRegistroK150');

        //var_dump('$this->_estrutResp', $this->_estrutResp); //die;

        $this->_nrpessoa['NRCPFPESSOA'] = null;
        $this->_nrpessoa['NMVINCULOM']  = null;
        $this->_nrpessoa['DTADMISSAOVINC'] = null;
        $this->_nrpessoa['NRESTRUTLEGAL']  = null;
        $this->_nrpessoa['NRVINCULOM'] = null;

        try{
            $params = ['P_NRORG'          => $this->_nrOrg,
                       'P_DTMESCOMPETENC' => $this->_dtCompetencia,
                       'P_DTPERIODINI'    => $this->_dtPeriodIni->format('d/m/Y'),
                       'P_DTPERIODFIM'    => $this->_dtPeriodFim->format('d/m/Y')];

            $sqlResult = $this->queryManad->get_regK150($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco K, Registro tipo K150 , item 4.4.'.self::ql_msg.'Error:'.$e->getMessage());
        }
        //var_dump('sqlResult', $sqlResult); die;
        
        foreach ($sqlResult as $regK150) {
            $cnpjcei = !empty($this->_estrutResp['CDCNPJ']) ? $this->_estrutResp['CDCNPJ'] : $this->_estrutResp['CDCEI'];

            $dtIncAlt = !empty($regK150['DTULTATU']) ? $regK150['DTULTATU'] : $regK150['DTINCLUSAO'];
            $cobRub  = $regK150['NREVENTOM'];
            $descRub = $regK150['NMEVENTOH'];
                
            $this->_blocoK->_registroK150->setCnpjCei($cnpjcei);
            $this->_blocoK->_registroK150->setDtIncAlt($dtIncAlt);
            $this->_blocoK->_registroK150->setCobRub($cobRub);
            $this->_blocoK->_registroK150->setDescRub($descRub);

            $this->_blocoK->_registroK150->geraRegistroK150();
            
            $registro = $this->_blocoK->_registroK150->getLinhaRegistro();
            $this->_addRegVinculo(Registro0000::REGISTRO_K150, $registro);

            $this->_blocoK->_registroK150->setLinhaRegistro();
    
            $this->_objMagnetico->fimAddRegVinculo();
            
            //die;
        }
    }

}