<?php

namespace HCM\FPA\ObrigacaoLegal\Manad;

use HCM\FPA\Entity\FpacalculofolhaEntity;

use HCM\FPA\ObrigacaoLegal\Manad\Blocos\Bloco0;
use HCM\FPA\ObrigacaoLegal\Manad\Blocos\BlocoI;
use HCM\FPA\ObrigacaoLegal\Manad\Blocos\Registros\Registro0000;
use HCM\FPA\ObrigacaoLegal\Manad\QueryManad;
use HCM\FPA\ObrigacaoLegal\Manad\Manad;
use HCM\FPA\Entity\FpaManadmagnetEntity;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\MsgCalculo;
use HCM\Util\Repositories;

class PercorreManad {

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_cdOperador;
    private $_dtCompetencia;
    private $_mesCompetencia;
    private $_calculoFolha;
    private $_manad;
    private $queryManad;
    private $_objMagnetico;

    private $_bloco0;
    private $_blocoI;
    
    const ql_msg = '<br>'; //Quebra de linha para as mensagens
    
    public function __construct($entityManager, $hcmParameters, $dataSet){
        $this->entityManager   = $entityManager;
        $this->hcmParameters   = $hcmParameters;
        $this->_nrOrg          = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao    = $hcmParameters->getNrorgPadrao();
        $this->_cdOperador     = $hcmParameters->getCdOperador();
        $this->_dtCompetencia  = $hcmParameters->getDtCompetencia();
        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);     
        $this->_calculoFolha   = new FpacalculofolhaEntity($entityManager, $hcmParameters);
        
        $this->_inicializaVariaveis($dataSet);

        $this->_objMagnetico = new FpaManadmagnetEntity($this->entityManager, $this->hcmParameters, $this->_nrOrg, $this->_mesCompetencia, $this->_nrInscEstrutResp);

        $tp_processamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch (strtoupper($tp_processamento)){
            case FPA::PROC_COMPETENCIA_MENSAL :
                $this->_retrocesso = false;
                break;
            case FPA::PROC_RETROCESSO :
                $this->_retrocesso = true;
                break;
            default:
                throw new \Exception('Tipo de processamento '.$tp_processamento.' inválido');
        }
    }

    private function _inicializaVariaveis($dataSet) {
        $this->_bloco0 = new Bloco0();
        $this->_blocoI = new BlocoI();

        $this->queryManad = new QueryManad($this->entityManager,$this->hcmParameters);

        $this->_buscaDadosFormulario($dataSet);

        $estruturaResp = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstrutura($this->_nrOrg, $this->_nrEstrResponsa, $this->_mesCompetencia);
        $this->_nrInscEstrutResp = $estruturaResp->historico->getCdcnpjestrut();
        
        // Calculo
        $this->_nrCalculoFolha = current($this->_calculoFolha->carregaListaNrCalculoFolhaTodasOcorrencias(
            $this->_nrOrg, 1, $this->_mesCompetencia, $this->hcmParameters->getTipoModalidadeCalculo()
        ));
        
        $this->beneficiariosManad = [];
        $this->registrosMagnetico = [];
        $this->dataEstrutResp = [];
    }
    
    //Função que carrega os dados do formulário. Na falha esta seta o nrOrg para -1 indicando a falha
    private function _buscaDadosFormulario($dataSet){
        
        $formObj = $this->entityManager->getRepository(Repositories::FPA_FORMMANAD)->findOneBy(array("nrorg" => $dataSet['NRORG'], "nrformmanad"=>$dataSet['NRFORMMANAD']));
        //var_dump('$formObj', $formObj);
        
        //Dados do formulário
        if(is_object($formObj)) {   //Somente grava os dados se encontrou o formulário segundo a Org e seu Nro
            $this->_nrForm         = $formObj->getNrformmanad();
            $this->_nmForm         = $formObj->getNmformmanad();
            $this->_idAtivo        = $formObj->getIdativo();
            $this->_dsOrdem        = $formObj->getDsordem();
            $this->_nomeInicial    = $formObj->getNomeinicial();
            $this->_nomeFinal      = $formObj->getNomefinal();
            $this->_dsInicio       = $formObj->getNomeinicial();
            $this->_dsFim          = $formObj->getNomefinal();
            $this->_idCentrEscrit  = $formObj->getIdcentrescrit();
            $this->_idFinaldArq    = $formObj->getIdfinaldarq();
            $this->_idGeraMagnetic = $formObj->getIdgerameiomag();
            $this->_idGeraArqv     = $formObj->getIdgeraarqv();
            $this->_nrPessResponsa = $formObj->getNrpessoarespons();
            $this->_nrEstrResponsa = $formObj->getNrestrutrespons();
            $this->_idTpInscriResp = $formObj->getIdtpinscrespons();
            $this->_nrGrupoInciden = $formObj->getNrgrpincide();
            $this->_dtPeriodIni    = $formObj->getDtperiodini();
            $this->_dtPeriodFim    = $formObj->getDtperiodfim();
            $this->_dtServInfoIni  = $formObj->getDtservinfoini();
            $this->_dtServInfoFim  = $formObj->getDtservinfofim();
            $this->_nrEstrutuLegal = $formObj->getNrestrutlegal();
            $this->_nrEstrutuGeren = $formObj->getNrestrutgeren();
            $this->_cdIniMatricula = $formObj->getCdinimatricula();
            $this->_cdFimMatricula = $formObj->getCdfimmatricula();
        } else {
            $this->_nrOrg = -1; //indica que não carregou os dados
            $this->msgErro = 'Formulário não carregado. O número do formulário '.$dataSet['NRFORMMANAD'].' da organização número '.$dataSet['NRORG'].' não foi encontrado.'.self::ql_msg;
        }
    }
    
    public function percorre(){
        //var_dump('aqui percorre manad');
        //die;

        if ($this->_retrocesso === false) {
            $resultado = $this->_geraMANAD();
            return $resultado;
        } else {            
            $cont = $this->_objMagnetico->excluirCompetencia();

            if($cont == 0) {
                $this->msgErro = "Não há registro de magnético a retroceder da Organização ".$this->_nrOrg." na competência ".$this->_dtCompetencia." e CNPJ da estrutura do responsável ".$this->_nrInscEstrutResp.self::ql_msg;
                return false;
            }
            return $cont;
        }
    }

    private function _geraMANAD(){
        var_dump('aqui _geraMANAD');

        //var_dump('P_NRTIPOESTRUTGEREN', $this->hcmParameters->getTipoEstruturaGerencialOrg());
        //var_dump('P_NRTIPOESTRUTOMADOR', $this->hcmParameters->getTipoEstruturaTomadorOrg()); die;
        
        //$this->_msg_idabort = false;    
        //$this->_msg_idstatus = MsgCalculo::ID_PENDENTE;
        $nrcalculomsg = $this->_nrCalculoFolha;
            
        $this->_preparaBloco0();
        $this->_preparaBlocoI(); 

        var_dump('Msgs de Erro/validações: ', $this->msgErro);
        //die;

        if ($this->_idGeraMagnetic == 'S') {

            $registros = $this->_objMagnetico->qdeRegistros();
            //var_dump('$registros', $registros); //die;

            if ($registros == 0) {

                if ($this->_idGeraArqv == 'S') {
                    $this->_objMagnetico->addRegCabecal($this->_nrInscEstrutResp, $this->_nrPessResponsa, 'S');
                }
                                    
                $vinculos = $this->retornaVinculosManad();

                if(empty($vinculos))
                    throw new \Exception('Não foram encontrados vínculos para o critério de seleção aplicado.');
                                
                //var_dump('$vinculos', $vinculos);
                var_dump('Foram encontrados '.count($vinculos).' vinculos');
                
                foreach ($vinculos as $vinculo) {
                    $beneficiarios[] = $vinculo['NRPESSOA'];
                }
                //var_dump('$beneficiarios', $beneficiarios);

                $this->_manad = new Manad($this->entityManager, $this->hcmParameters, $this->_objMagnetico, $this->dataEstrutResp, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim,
                                          $this->_dtPeriodIni,  $this->_dtPeriodFim);

                $this->_manad->_geraRegistroK001(); //Abertura bloco k

                var_dump('Continuação dos registros K na tabela FPA_MANADMAGNET.DSLINHAREGISTRO.');
                
                foreach ($beneficiarios as $nrpessoa) {
                    
                    $this->_manad->geraManad($nrpessoa);

                    /*$processamento = MsgCalculo::getInstance();
                    $processamento->setDados($nrcalculomsg, $nrpessoa, $this->_nrOrg, 'DIRF');  
                    if (!is_null($nrcpfbeneficiario)) {
                        $this->beneficiariosManad[$nrcpfbeneficiario] = 1; //Armazena os CPFs dos participantes da DIRF
                    }*/
                }
                
                $this->_manad->_preparaRegistroK150(); //registro geral fora de vínculo.

            } else {
                throw new \Exception('Já existe magnético da MANAD para o mês de competência, favor retroceder.');
            }
        }
        var_dump('Fim gravação magnético!');
        //die;
        
        $this->setRegistrosMagnetico();
        
        /*if ($this->_idGeraMagnetic == 'S' || $this->_idGeraArqv == 'S') {
            $this->_preparaRegistroPSE();
        }*/
        
        //if (($this->_msg_idabort == false) && ($this->_msg_idstatus !== MsgCalculo::ID_COM_ERRO)) {
            if ($this->_idGeraArqv == 'S') {
                $arquivo = $this->_geraMagnetico();
            }
        //}
        
        return [ 'ARQUIVO' => $arquivo,
                 'CALCULO' => $nrcalculomsg,
                 'ERRO'    => $this->getMensagens(), //(($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) || ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)),
                 'ABORT'   => $this->_msg_idabort];
    }

    private function _geraMagnetico(){

        $nomeArquivo = "MANAD_".$this->_nrInscEstrutResp.".txt";

        $this->_bloco0->_registro0000->geraRegistro0000();
        FPA::escreveArquivo($nomeArquivo, $this->_registro0000->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF, false, true);

        $this->_bloco0->_registro0001->geraRegistro0001(); 
        FPA::escreveArquivo($nomeArquivo, $this->_registro0001->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF, false, true);

        
        // CONTINUAR A CRIAR O TXT !!!


        /*$this->_registro0000->geraRegistroDIRF();
        FPA::escreveArquivo($nomeArquivo, $this->_registro0000->getLinhaRegistro().RegistroDIRF::QUEBRA_LINHA_CR_LF, false, true);

        $this->_registroRESPO->geraRegistroRESPO();
        FPA::escreveArquivo($nomeArquivo, $this->_registroRESPO->getLinhaRegistro().RegistroDIRF::QUEBRA_LINHA_CR_LF);

        $this->_registroDECPJ->geraRegistroDECPJ();
        FPA::escreveArquivo($nomeArquivo, $this->_registroDECPJ->getLinhaRegistro().RegistroDIRF::QUEBRA_LINHA_CR_LF);*/
        
        $idrecAnt = -1;
        //$registros = $this->_objMagnetico->retornaRegistros();
        foreach ($this->registrosMagnetico as $item) {
            
            //Se estiver marcado no formulário para somente vinculos com rendimento superior a X
            if ($this->_idVincRendimento == 'S') {
                if ($item->getVrtotalrend() < $this->_vrMinRendimento) {
                    continue;
                }
            }
            
            //Se estiver marcado no formulário para somente vinculos com retenção
            if ($this->_idVincRetencao == 'S') {
                if ($item->getVrtotimpretido() == 0) {
                    continue;
                }
            }
            
            $idrec = substr($item->getDslinharegistro(),0,strpos($item->getDslinharegistro(),"\nBPFDEC"));
            $registro = substr($item->getDslinharegistro(),strpos($item->getDslinharegistro(),"\nBPFDEC")+1,strlen($item->getDslinharegistro()));

            if ($idrec != $idrecAnt) {
                $idrecAnt = $idrec;
                FPA::escreveArquivo($nomeArquivo, $idrec.RegistroDIRF::QUEBRA_LINHA_CR_LF);
            }

            FPA::escreveArquivo($nomeArquivo, $registro.RegistroDIRF::QUEBRA_LINHA_CR_LF);
        }
        
        foreach($this->_registroPSE as $registro) {
            FPA::escreveArquivo($nomeArquivo, $registro.RegistroDIRF::QUEBRA_LINHA_CR_LF);
        }

        FPA::escreveArquivo($nomeArquivo, $this->_fimDirf().RegistroDIRF::QUEBRA_LINHA_CR_LF);
        
        return $nomeArquivo;
    }

    private function _preparaBloco0(){
        var_dump('aqui _preparaBloco0');
        
        $this->_preparaRegistro0000();
            $this->_bloco0->_registro0000->geraRegistro0000(); //die; //REMOVER!!!
            $this->_bloco0->_registro0001->geraRegistro0001(); //REMOVER!!!
        $this->_preparaRegistro0050();
            $this->_bloco0->_registro0050->geraRegistro0050(); //REMOVER!!!
        
        $this->_preparaRegistro0100();
            $this->_bloco0->_registro0100->geraRegistro0100(); //REMOVER!!!
            $this->_bloco0->_registro0990->geraRegistro0990(); //REMOVER!!!
    }

    private function _preparaBlocoI(){
        var_dump('aqui _preparaBlocoI');

        $this->_blocoI->_registroI001->geraRegistroI001(); //REMOVER!!!
        $this->_blocoI->_registroI990->geraRegistroI990(); //REMOVER!!!
    }

    private function _preparaRegistro0000(){
        var_dump('aqui _preparaRegistro0000');
        
        $cpf = null; $cnpj = null;

        try{
            $params = ['P_NRORG'                  => $this->_nrOrg,
                       'P_ESTRUTURARESPONSAVEL'   => $this->_nrEstrResponsa,
                       'P_DTMESCOMPETENC'         => $this->_dtCompetencia];
            
            $sqlResult = $this->queryManad->get_reg0000($params);
            $this->dataEstrutResp = $sqlResult;
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco 0, Registro tipo 0000, item 4.2.'.self::ql_msg.'Error:'.$e->getMessage());
        }
        //var_dump('sqlResult', $sqlResult); //die;

        if(empty($sqlResult)) {
            $this->msgErro .= "Sem dados no Bloco 0, Registro tipo 0000, item 4.2.".self::ql_msg;
        } else {
            if(empty($sqlResult['NOMEEMPRESARIAL'])) {
                $this->msgErro .= "Está faltando o nome empresarial do contribuinte.".self::ql_msg;
            }
            if(empty($sqlResult['CDUF'])) {
                $this->msgErro .= "Está faltando a unidade da federação do contribuinte.".self::ql_msg;
            }
            if(empty($sqlResult['CDMUN'])) {
                $this->msgErro .= "Está faltando Código do município do domicílio fiscal do contribuinte.".self::ql_msg;
            }
            if(empty($sqlResult['CDCNPJ'])) {
                $this->msgErro .= "Está faltando o cpf ou cnpj do contribuinte.".self::ql_msg;
            }
            else {
                if($sqlResult['TPINSC'] == 'CNPJ')
                    $cnpj = $sqlResult['CDCNPJ'];
                else
                    $cpf = substr($sqlResult['CDCNPJ'], 0, 11);
            }
            if(empty($this->_dtPeriodIni) || empty($this->_dtPeriodFim)) {
                $this->msgErro .= "Está faltando o período inicial ou final.".self::ql_msg;
            }
            if(is_null($this->_idCentrEscrit)){
                $this->msgErro .= "Está faltando a indicação da Centralização de Escrituração.".self::ql_msg;
            }
            if(empty($this->_idFinaldArq)){
                $this->msgErro .= "Está faltando a indicação da Finalidade do Arquivo.".self::ql_msg;
            }
            
            $nome = $sqlResult['NOMEEMPRESARIAL'];
            $cei  = $sqlResult['CDCEI'];
            $dtIni = $this->_dtPeriodIni->format('d/m/Y');
            $dtFim = $this->_dtPeriodFim->format('d/m/Y');
            $uf = $sqlResult['CDUF'];
            $codMun = $sqlResult['CDMUN'];
                
            $this->_bloco0->_registro0000->setNome($nome);
            $this->_bloco0->_registro0000->setCnpj($cnpj);
            $this->_bloco0->_registro0000->setCpf($cpf);
            $this->_bloco0->_registro0000->setCei($cei);
            $this->_bloco0->_registro0000->setNit(null);
            $this->_bloco0->_registro0000->setUf($uf);
            $this->_bloco0->_registro0000->setIe(null);
            $this->_bloco0->_registro0000->setCodMun($codMun);
            $this->_bloco0->_registro0000->setIm(null);
            $this->_bloco0->_registro0000->setSuframa(null);
            $this->_bloco0->_registro0000->setIndCentr($this->_idCentrEscrit);
            $this->_bloco0->_registro0000->setDtIni($dtIni);
            $this->_bloco0->_registro0000->setDtFim($dtFim);
            $this->_bloco0->_registro0000->setCodFin($this->_idFinaldArq);
        }
    }

    private function _preparaRegistro0050(){
        var_dump('aqui _preparaRegistro0050');
        
        try{
            $params = ['P_NRORG'                  => $this->_nrOrg,
                       'P_ESTRUTURARESPONSAVEL'   => $this->_nrEstrResponsa,
                       'P_DTMESCOMPETENC'         => $this->_dtCompetencia,
                       'P_NRORGPADRAO'            => $this->_nrOrgPadrao,
                       'P_NRPESSRESPONSA'         => $this->_nrPessResponsa];

            $sqlResult = $this->queryManad->get_reg0050($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco 0, Registro tipo 0050, item 4.2.'.self::ql_msg.'Error:'.$e->getMessage());
        }
        //var_dump('sqlResult', $sqlResult);

        if(empty($sqlResult)) {
            $this->msgErro .= "Sem dados no Bloco 0, Registro tipo 0050, item 4.2.".self::ql_msg;
        } else {
            if(empty($sqlResult['NMNOMERESP'])) {
                $this->msgErro .= "Está faltando o nome do responsável contabilista/escritório de contabilidade.".self::ql_msg;
            }
            if(empty($sqlResult['CDCNPJ'])) {
                $this->msgErro .= "Está faltando o cnpj do escritório de contabilidade.".self::ql_msg;
            }
            if(empty($sqlResult['CDCPF'])) {
                $this->msgErro .= "Está faltando o cpf do responsável contabilista.".self::ql_msg;
            }
            if(empty($sqlResult['CDCRC'])) {
                $this->msgErro .= "Está faltando o número de inscrição do responsável contabilista no Conselho Regional de Contabilidade.".self::ql_msg;
            }
            
            /* VER SE PRECISA ADD MAIS VALIDAÇÕES */
            
            $nome = $sqlResult['NMNOMERESP'];
            $cnpj = $sqlResult['CDCNPJ'];
            $cpf  = $sqlResult['CDCPF'];
            $crc  = $sqlResult['CDCRC'];
            $dtIni = $this->_dtPeriodIni->format('d/m/Y');
            $dtFim = $this->_dtPeriodFim->format('d/m/Y');
            $end = $sqlResult['DSEND'];
            $num = $sqlResult['NRNUMEND'];
            $compl  = $sqlResult['DSCOMP'];
            $bairro = $sqlResult['DSBAIRRO'];
            $cep = $sqlResult['CDCEP'];
            $uf  = $sqlResult['SGESTADO'];
            $fone = trim($sqlResult['NRDDD'].' '.$sqlResult['NRTELEFONE'], ' ');
            $fax  = trim($sqlResult['NRDDDFAX'].' '.$sqlResult['NRFAX'], ' ');
            $email = $sqlResult['NMEMAIL'];
                
            $this->_bloco0->_registro0050->setNome($nome);
            $this->_bloco0->_registro0050->setCnpj($cnpj);
            $this->_bloco0->_registro0050->setCpf($cpf);
            $this->_bloco0->_registro0050->setCrc($crc);
            $this->_bloco0->_registro0050->setDtIni($dtIni);
            $this->_bloco0->_registro0050->setDtFim($dtFim);
            $this->_bloco0->_registro0050->setEnd($end);
            $this->_bloco0->_registro0050->setNum($num);
            $this->_bloco0->_registro0050->setCompl($compl);
            $this->_bloco0->_registro0050->setBairro($bairro);
            $this->_bloco0->_registro0050->setCep($cep);
            $this->_bloco0->_registro0050->setUf($uf);
            $this->_bloco0->_registro0050->setFone($fone);
            $this->_bloco0->_registro0050->setFax($fax);
            $this->_bloco0->_registro0050->setEmail($email);
        }
    }

    private function _preparaRegistro0100(){
        var_dump('aqui _preparaRegistro0100');
        
        try{
            $params = ['P_NRORG'                  => $this->_nrOrg,
                       'P_ESTRUTURARESPONSAVEL'   => $this->_nrEstrResponsa,
                       'P_DTMESCOMPETENC'         => $this->_dtCompetencia,
                       'P_NRPESSRESPONSA'         => $this->_nrPessResponsa];

            $sqlResult = $this->queryManad->get_reg0100($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco 0, Registro tipo 0100, item 4.2.'.self::ql_msg.'Error:'.$e->getMessage());
        }
        //var_dump('sqlResult', $sqlResult); die;

        if(empty($sqlResult)) {
            $this->msgErro .= "Sem dados no Bloco 0, Registro tipo 0100, item 4.2.".self::ql_msg;
        } else {
            if(empty($sqlResult['NMNOMERESP'])) {
                $this->msgErro .= "Está faltando o nome da empresa/técnico responsável pela geração do arquivo.".self::ql_msg;
            }
            if(empty($sqlResult['NMCARGO'])) {
                $this->msgErro .= "Está faltando o cargo do responsável na empresa.".self::ql_msg;
            }

            if(empty($sqlResult['CDCNPJ'])) {
                $this->msgErro .= "Está faltando o cnpj da empresa responsável pela geração do arquivo.".self::ql_msg;
            }
            if(empty($sqlResult['CDCPF'])) {
                $this->msgErro .= "Está faltando o cpf do técnico responsável pela geração do arquivo.".self::ql_msg;
            }
            if(empty($this->_dtServInfoIni) || empty($this->_dtServInfoFim)) {
                $this->msgErro .= "Está faltando o período da prestação do serviço de informática.".self::ql_msg;
            }
            
                /* VER SE PRECISA ADD MAIS VALIDAÇÕES */
            
            $nome  = $sqlResult['NMNOMERESP'];
            $cargo = $sqlResult['NMCARGO'];
            $dtIni = $this->_dtServInfoIni->format('d/m/Y');
            $dtFim = $this->_dtServInfoFim->format('d/m/Y');
            $cnpj = $sqlResult['CDCNPJ'];
            $cpf  = $sqlResult['CDCPF'];
            $fone = trim($sqlResult['NRDDD'].' '.$sqlResult['NRTELEFONE'], ' ');
            $fax  = trim($sqlResult['NRDDDFAX'].' '.$sqlResult['NRFAX'], ' ');
            $email = $sqlResult['NMEMAIL'];
                
            $this->_bloco0->_registro0100->setNome($nome);
            $this->_bloco0->_registro0100->setCargo($cargo);
            $this->_bloco0->_registro0100->setDtIni($dtIni);
            $this->_bloco0->_registro0100->setDtFim($dtFim);
            $this->_bloco0->_registro0100->setCnpj($cnpj);
            $this->_bloco0->_registro0100->setCpf($cpf);
            $this->_bloco0->_registro0100->setFone($fone);
            $this->_bloco0->_registro0100->setFax($fax);
            $this->_bloco0->_registro0100->setEmail($email);
        }
    }
    
    private function retornaVinculosManad() {
        //var_dump('aqui retornaVinculosManad');

        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($this->_mesCompetencia));
        $primeiroDiaAno = DateUtil::getPrimeiroDiaDoAno(DateUtil::getDataDeString($this->_mesCompetencia));
        $primeiroDiaAno = DateUtil::addSubIntervalo($primeiroDiaAno, -1, DateUtil::MES);
        
        $params = ['P_DS_INICIO'       => $this->_dsInicio,
                   'P_DS_FIM'          => $this->_dsFim,
                   'P_DSORDEM'         => $this->_dsOrdem,
                   'P_PRIMEIRODIAANO'  => $primeiroDiaAno->format('d/m/Y'),
                   'P_DTULTDIAMES'     => $dtUltDiaMes->format('d/m/Y'),
                   'P_NRORG'           => $this->_nrOrg];
        
        try{
            return $this->queryManad->montaSQLVinculoManad($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query montaSQLVinculoManad: Bloco K - Folha de Pagamento.'.self::ql_msg.'Error:'.$e->getMessage());
        }
    }

    public function getMensagens(){
        return is_null($this->msgErro) ? false : $this->msgErro;
    }

    private function _preparaRegistroPSE(){
        
        try{
            $params = array(
                "nrorg"             => $this->_nrOrg,
                "inicio"            => $this->_dsInicio,
                "fim"               => $this->_dsFim,
                "dtcompetencia"     => $this->_dtCompetencia,
                "nrgrupoinciden"    => $this->_nrGrupoInciden,
                "nrcenario"         => 1,
            );
            
            switch($this->_dsOrdem){ 
                case 'ALFABETICA':
                    $this->queryPart = "AND (VM.NMVINCULOM BETWEEN :inicio AND :fim)";
                    break;
                case 'DATA_ADMISSAO':
                    $this->queryPart = "AND (VM.DTADMISSAOVINC BETWEEN :inicio AND :fim)";
                    break;
                case 'MATRICULA':
                    $this->queryPart = "AND (VM.CDMATRICULA BETWEEN :inicio AND :fim)";
                    break;
                case 'ESTRUTURA':
                    if ($this->_dsFim) {
                        $this->queryPart = "AND (VH.NRESTRUTLEGAL = :inicio AND VH.NRESTRUTGEREN = :fim)";
                    } else {
                        $this->queryPart = "AND (VH.NRESTRUTLEGAL = :inicio AND VH.NRESTRUTGEREN  = NVL('', VH.NRESTRUTGEREN))";
                    }
                    break;
                case 'LISTA_VINCULO':
                    $inicio = is_array($this->_dsInicio) ? implode(',',$this->_dsInicio) : $this->_dsInicio;
                    $this->queryPart = "AND VM.NRVINCULOM IN ($inicio)";
                    break;
                case 'LISTA_ESTRUTURA':
                    $inicio = is_array($this->_dsInicio) ? implode(',',$this->_dsInicio) : $this->_dsInicio;
                    $this->queryPart = "AND VH.NRESTRUTLEGAL IN ($inicio)";
                    if($fim){
                        $fim = is_array($this->_dsFim) ? implode(',',$this->_dsFim) : $this->_dsFim;
                        $this->queryPart .= "AND VH.NRESTRUTGEREN IN ($fim)";
                    }
                    break; 
                default:
                    throw new \Exception( "Opção de ordenação inválida (".$this->_dsOrdem.").");
            }
        
            /*Ajusta a query para regime competência ou caixa*/
            // if($this->_nrRegimeApur == '2') {    //Regime Caixa
            //     $queryPartPse = "AND CF.DTOCORRENCIA BETWEEN ADD_MONTHS(TRUNC(TO_DATE( :dtcompetencia ), 'YEAR'), -1) AND ADD_MONTHS( :dtcompetencia, -1)";
            // } else {
                $queryPartPse = "AND CF.DTOCORRENCIA BETWEEN TRUNC( TO_DATE( :dtcompetencia ), 'YEAR' ) AND :dtcompetencia";
            // }
            
            if($this->_idApuraBeneficio == 'S'){
                $sqlResult = $this->queryDirf->get_pse_apurabeneficio($params,$this->queryPart,$queryPartPse);
            } else {
                $sqlResult = $this->queryDirf->get_pse($params,$this->queryPart,$queryPartPse);
            }
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo PSE, item 5. ('.$e->getMessage().')');
        }
        
        $pse = '';
        $opseAntes  = '';    //Armazena os valores anteriores
        $cnpjOpseAntes = '';
        $tpseAntes = '';      
        $cpfTitular = '';
        $this->_registroPSE = array();
        
        foreach($sqlResult as $val){
            
            if(isset($this->beneficiariosManad[$val['NRCPFTITULAR']])) {
            //if(isset($this->registrosMagnetico[$val['NRCPFTITULAR']])) {
                
                if ($pse == '') {
                    $pse = $val['PSE'];
                    array_push($this->_registroPSE, $pse);    //Adiciona o registro PSE (somente uma vez)
                }
                
                $opse = $val['OPSE'] . $val['CNPJ_EMPR'] . $val['NOME_EMPR'] . $val['REG_ANS'];
                $cnpjOpse = $val['CNPJ_EMPR'];
                $tpse = $val['TPSE'] . $val['CDCPF'] . $val['NOME'] . $val['VALOR'];
                $tpseref = $val['TPSE'] . $val['CDCPF'] . $val['NOME'];
                $dtpse = $val['DTPSE'] . $val['CDCPFDEP'] . $val['DTNASCDEP'] . $val['NOMEDEP'] . $val['RELACDEP'] . $val['VALORDEP'];
                $isDep   = $val['NOMEDEP'] != '|';      //Flag para indicação que o TPSE / registro atual se trata de um dependente
                
//                if ($opse != $opseAntes) {              //Verifica se houve novo registro do tipo OPSE
                if ($cnpjOpse != $cnpjOpseAntes) {              //Verifica se houve novo registro do tipo OPSE
                    if ($val['CDCNPJESTRUT'] == '|')    //'IFs' para verificação de campos não preenchidos
                        $this->msgErro .= "Estrutura (" . $val['NMRAZSOCESTRUT'] . ") sem CNPJ." . self::ql_msg;
//                    $opseAntes = $opse;                 //Atualiza o OPSE corrente
                    $cnpjOpseAntes = $cnpjOpse;         //Atualiza o OPSE corrente
                    $tpseAntes = '';                    //Força a utilização do TPSE já que o OPSE mudou
                    array_push($this->_registroPSE, $opse);//Adiciona o registro OPSE
                }
    
                if ($tpse != $tpseAntes && !$isDep) {       //Verifica se houve novo registro do tipo TPSE para TITULAR
                    $tpseAntes  = $tpse;                    //Atualiza o TPSE corrente
                    $cpfTitular = $val['NRCPFTITULAR'];     //Registra CPF do titular para identificação na parte de dependentes DTPSE
                    array_push($this->_registroPSE, $tpse);          //Adiciona o registro TPSE
                }
    
                if ($isDep && $cpfTitular == $val['NRCPFTITULAR']) {    //É dependente do titular corrente?
                    array_push($this->_registroPSE, $dtpse); 
                }

                if ($this->_idGeraArqv == 'S') {
                    $dsRegistro = $val['NOMEDEP'] == '|' ? $opse . $tpseAntes: $opse . $tpseAntes . $dtpse;
                        
                    $this->_objMagnetico->setAllToPSEObj($val['NRCPFTITULAR'], $dsRegistro, str_replace('|', '', $val['NOME']));
                }
            }

        }
        
        if (!empty($this->_registroPSE)) { //Caso tenha gerado plano(s) de saúde altera o identificador do registro DECPJ
            $this->_registroDECPJ->setIdPagPlanoSaud('S');
        }
    }
    
    private function setRegistrosMagnetico(){
        var_dump('aqui setRegistrosMagnetico');

        $registros = $this->_objMagnetico->retornaRegistros();

        foreach ($registros as $registro) {

            $this->registrosMagnetico[$registro->getIdtiporegistro()][] = $registro;
                        
            //$this->beneficiariosManad[$registro->getNrcpfpessoa()] = 1;
        }

        //var_dump('$this->registrosMagnetico', $this->registrosMagnetico);

        var_dump('FIM GERAÇÃO PARCIAL MANAD !!!'); die;

    }
    
    private function _fimDirf(){
        return 'FIMDIRF|';
    }
    
    public function geraRelatorio(){
        $filtro = array(
            'P_COMPETENCIA'=> $this->_dtCompetencia,
            'P_NRORG'      => $this->_nrOrg,
            //'P_ANOREF'     => $this->_anoReferencia,
            'P_ANOCALEN'   => $this->_anoCalendario,
            'P_VINCULO'    => '', /*$vinculos_str,*/
            'P_ESTRUTURARESP' => $this->_nrEstrResponsa
        );
        return $filtro; 
    }
    
    public function geraRelatorioAnalitico() {
        $filtro = array(
            'P_COMPETENCIA'   => $this->_dtCompetencia,
            'P_NRORG'         => $this->_nrOrg,
            'P_VINCULO'       => '', /*$vinculos_str,*/
            'P_ESTRUTURARESP' => $this->_nrEstrResponsa
        );
        return $filtro;
        
    }

}