<?php
namespace HCM\FPA\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\FPA;

class IteCalcFolhaRepository {

    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
    }

    public function getItensCalc($nrcalculobenef, $nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgeren, $nrestrutura_beneficio, $lista_nrvinculom)
    {

        $params = [
            'NRORG'                 => $nrorg,
            'NRESTRUTLEGAL'         => $nrestrutlegal,
            'NRESTRUTGEREN'         => $nrestrutgeren,
            'DTCOMPETENC'          => $dtmescompetenc,
            'NRCALCULOFOLHA'        => $nrcalculobenef,
            'LISTA_NRVINCULOM'      => $lista_nrvinculom,
            'NRESTRUTURA_BENEFICIO' => $nrestrutura_beneficio
        ];
 
        $sql = <<<SQL
    SELECT      
        FPA_ITECALCFOLHA.NRITEMCALCFOLHA,
        FPA_ITECALCFOLHA.NRCALCULOFOLHA,
        FPA_ITECALCFOLHA.NRVINCULOM,
        FPA_ITECALCFOLHA.NREVENTOM,
        FPA_ITECALCFOLHA.NRORG,
        FPA_ITECALCFOLHA.IDEVENDEMONCALC,
        FPA_ITECALCFOLHA.VRREFFOLHA AS VRREFFOLHALOTE,
        FPA_ITECALCFOLHA.VREVENTOFOLHA AS VREVENTOLOTE,
        FPA_ITECALCFOLHA.QTOCORPROXPERI,
        FPA_ITECALCFOLHA.IDREFEVALORCALC,
        FPA_ITECALCFOLHA.DTINCLUSAO,
        FPA_ITECALCFOLHA.DTULTATU,
        FPA_ITECALCFOLHA.NRORGINCLUSAO,
        FPA_ITECALCFOLHA.CDOPERINCLUSAO,
        FPA_ITECALCFOLHA.DTOCORRENCIA,
        FPA_ITECALCFOLHA.VRVELHO,
        FPA_ITECALCFOLHA.NRLOTELANCEVENTO,
        FPA_ITECALCFOLHA.DTGERULTCNAB,
        FPA_ITECALCFOLHA.DSITEMCALCFOLHA
    FROM 
        FPA_ITECALCFOLHA
    JOIN 
        GPE_VINCULOH
    ON 
    (
        GPE_VINCULOH.NRORG = FPA_ITECALCFOLHA.NRORG
        AND GPE_VINCULOH.NRVINCULOM = FPA_ITECALCFOLHA.NRVINCULOM
        AND (GPE_VINCULOH.NRESTRUTLEGAL = :NRESTRUTLEGAL OR :NRESTRUTLEGAL IS NULL)
        AND (GPE_VINCULOH.NRESTRUTGEREN = :NRESTRUTGEREN OR :NRESTRUTGEREN IS NULL)
        AND (INSTR(:LISTA_NRVINCULOM, '_'||GPE_VINCULOH.NRVINCULOM||'_') > 0 OR :LISTA_NRVINCULOM = 'A')
        AND GPE_VINCULOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                           WHERE H1.NRORG = GPE_VINCULOH.NRORG
                                           AND H1.NRVINCULOM = GPE_VINCULOH.NRVINCULOM
                                           AND H1.DTMESCOMPETENC <= :DTCOMPETENC)
    )


    WHERE FPA_ITECALCFOLHA.NRORG = :NRORG
        AND FPA_ITECALCFOLHA.DTOCORRENCIA BETWEEN :DTCOMPETENC AND LAST_DAY(TO_DATE(:DTCOMPETENC, 'DD/MM/YYYY'))
        AND FPA_ITECALCFOLHA.NRCALCULOFOLHA = :NRCALCULOFOLHA
        AND FPA_ITECALCFOLHA.NREVENTOM NOT IN (3030, 3040)
        AND (
            EXISTS (
                SELECT * FROM
                    VAB_BENEFICIARIO 
    
                JOIN
                    VAB_BENEFICIOM BNM 
                        ON(
                            VAB_BENEFICIARIO.NRBENEFICIOM = BNM.NRBENEFICIOM 
                            AND VAB_BENEFICIARIO.NRORG = BNM.NRORG 
                        ) 
                JOIN
                    VAB_BENEFICIOH BNH 
                        ON(
                            BNM.NRBENEFICIOM = BNH.NRBENEFICIOM 
                            AND BNM.NRORG = BNH.NRORG
                            AND (BNH.NRESTRUTURAM = :NRESTRUTURA_BENEFICIO)
                            AND BNH.DTMESCOMPETENC =(
                                SELECT
                                    MAX(BNEH.DTMESCOMPETENC) 
                            FROM
                                VAB_BENEFICIOH BNEH 
                            WHERE
                                BNEH.NRORG = BNH.NRORG 
                                AND BNEH.NRBENEFICIOM = BNH.NRBENEFICIOM 
                                AND BNEH.DTMESCOMPETENC <= :DTCOMPETENC 
                        ) 
                    )
                WHERE(
                    FPA_ITECALCFOLHA.NRVINCULOM = VAB_BENEFICIARIO.NRVINCULOM 
                    AND FPA_ITECALCFOLHA.NRORG = VAB_BENEFICIARIO.NRORG 
                ) 
            )
            OR :NRESTRUTURA_BENEFICIO IS NULL
        )
SQL;
        
        $result = $this->connection->fetchAll($sql, $params);
        
        return $result;
    }

    public function getItensCalcListas($nrcalculobenef, $nrorg, $dtmescompetenc, $lista_nrestrutlegal, $lista_nrestrutgeren, $lista_nrestrutsind, $lista_nrvinculom,
                                       $lista_nrestrutbenef, $lista_nrbeneficiom, $lista_nrtpbeneficio, $lista_nreventom)
    {

        $params = [
            'NRCALCULOBENEF' => $nrcalculobenef,
            'NRORG' => $nrorg,
            'DTMESCOMPETENC' => $dtmescompetenc,
            'LISTA_NRESTRUTLEGAL' => $lista_nrestrutlegal,
            'LISTA_NRESTRUTGEREN' => $lista_nrestrutgeren,
            'LISTA_NRESTRUTSIND' => $lista_nrestrutsind,
            'LISTA_NRVINCULOM' => $lista_nrvinculom,
            'LISTA_NRESTRUTBENEF' => $lista_nrestrutbenef,
            'LISTA_NRBENEFICIOM' => $lista_nrbeneficiom,
            'LISTA_NRTPBENEFICIO' => $lista_nrtpbeneficio,
            'LISTA_NREVENTOM' => $lista_nreventom,
            'P_NRESTRUTLEGALOPER' => FPA::getNrestrutlegalOper(),
            'P_NRESTRUTGERENOPER' => FPA::getNrestrutgerenOper()
        ];
 
        $sql = <<<SQL
    SELECT 
        FPA_ITECALCFOLHA.NRVINCULOM,
        FPA_ITECALCFOLHA.NREVENTOM,
        FPA_ITECALCFOLHA.NRORG,
        SUM(FPA_ITECALCFOLHA.VRREFFOLHA) AS VRREFFOLHALOTE,
        SUM(FPA_ITECALCFOLHA.VREVENTOFOLHA) AS VREVENTOLOTE
    FROM 
        FPA_ITECALCFOLHA
    JOIN 
        GPE_VINCULOH
    ON 
    (
        GPE_VINCULOH.NRORG = FPA_ITECALCFOLHA.NRORG
        AND GPE_VINCULOH.NRVINCULOM = FPA_ITECALCFOLHA.NRVINCULOM
        AND (INSTR(:LISTA_NRESTRUTLEGAL, '_'||GPE_VINCULOH.NRESTRUTLEGAL||'_') > 0 OR :LISTA_NRESTRUTLEGAL = 'A')
        AND (INSTR(:LISTA_NRESTRUTGEREN, '_'||GPE_VINCULOH.NRESTRUTGEREN||'_') > 0 OR :LISTA_NRESTRUTGEREN = 'A')
        AND (INSTR(:LISTA_NRESTRUTSIND, '_'||GPE_VINCULOH.NRESTRUTSIND||'_') > 0 OR :LISTA_NRESTRUTSIND = 'A')
        AND (INSTR(:LISTA_NRVINCULOM, '_'||GPE_VINCULOH.NRVINCULOM||'_') > 0 OR :LISTA_NRVINCULOM = 'A')
        AND GPE_VINCULOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                           WHERE H1.NRORG = GPE_VINCULOH.NRORG
                                           AND H1.NRVINCULOM = GPE_VINCULOH.NRVINCULOM
                                           AND H1.DTMESCOMPETENC <= :DTMESCOMPETENC)
        AND ( INSTR(','
                    || :P_NRESTRUTLEGALOPER
                    || ',', ','
                            || GPE_VINCULOH.NRESTRUTLEGAL
                            || ',') > 0
              OR :P_NRESTRUTLEGALOPER IS NULL )
        AND ( INSTR(','
                    || :P_NRESTRUTGERENOPER
                    || ',', ','
                            || GPE_VINCULOH.NRESTRUTGEREN
                            || ',') > 0
              OR :P_NRESTRUTGERENOPER IS NULL )
    )

    WHERE FPA_ITECALCFOLHA.NRORG = :NRORG
        AND FPA_ITECALCFOLHA.NRCALCULOFOLHA = :NRCALCULOBENEF
        AND FPA_ITECALCFOLHA.NREVENTOM NOT IN (3030, 3040)
        AND (INSTR(:LISTA_NREVENTOM, '_'||FPA_ITECALCFOLHA.NREVENTOM||'_') > 0 OR :LISTA_NREVENTOM = 'A')
        AND (
            EXISTS (
                SELECT * FROM
                    VAB_BENEFICIARIO 
    
                JOIN
                    VAB_BENEFICIOM BNM 
                        ON(
                            VAB_BENEFICIARIO.NRBENEFICIOM = BNM.NRBENEFICIOM 
                            AND VAB_BENEFICIARIO.NRORG = BNM.NRORG 
                        ) 
                JOIN
                    VAB_BENEFICIOH BNH 
                        ON(
                            BNM.NRBENEFICIOM = BNH.NRBENEFICIOM 
                            AND BNM.NRORG = BNH.NRORG
                            AND (INSTR(:LISTA_NRESTRUTBENEF, '_'||BNH.NRESTRUTURAM||'_') > 0 OR :LISTA_NRESTRUTBENEF = 'A')
                            AND (INSTR(:LISTA_NRTPBENEFICIO, '_'||BNH.NRTPBENEFICIO||'_') > 0 OR :LISTA_NRTPBENEFICIO = 'A')
                            AND (INSTR(:LISTA_NRBENEFICIOM, '_'||BNH.NRBENEFICIOM||'_') > 0 OR :LISTA_NRBENEFICIOM = 'A')
                            AND BNH.DTMESCOMPETENC =(
                                SELECT
                                    MAX(BNEH.DTMESCOMPETENC) 
                            FROM
                                VAB_BENEFICIOH BNEH 
                            WHERE
                                BNEH.NRORG = BNH.NRORG 
                                AND BNEH.NRBENEFICIOM = BNH.NRBENEFICIOM 
                                AND BNEH.DTMESCOMPETENC <= :DTMESCOMPETENC 
                        ) 
                    )
                WHERE(
                    FPA_ITECALCFOLHA.NRVINCULOM = VAB_BENEFICIARIO.NRVINCULOM 
                    AND FPA_ITECALCFOLHA.NRORG = VAB_BENEFICIARIO.NRORG 
                    AND VAB_BENEFICIARIO.DTCANCBENEFICIARIO IS NULL
                ) 
            )
        )
        GROUP BY FPA_ITECALCFOLHA.NRVINCULOM,
        FPA_ITECALCFOLHA.NREVENTOM,
        FPA_ITECALCFOLHA.NRORG
SQL;
        $result = $this->connection->fetchAll($sql, $params);
        
        return $result;
    }

} 
