<?php

namespace HCM\FPA\Services;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Doctrine\DBAL\Connection;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use Doctrine\ORM\EntityManager;
use HCM\FPA\ObrigacaoLegal\CNAB\Banestes\Cnab021;
use HCM\FPA\ObrigacaoLegal\CNAB\Itau\Cnab341;
use HCM\FPA\ObrigacaoLegal\CNAB\Caixa\Cnab104;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab240;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab200;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab240237;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab200237;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\Messages;
use HCM\Util\Util;


class CNABLeituraService {

    public function __construct (EntityManager $entityManager, HCMParameters $hcmParameters, Environment $environment, Kernel $httpKernel){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->httpKernel = $httpKernel;
        $this->hcmParameters = $hcmParameters;
    }

    public function leituraArquivoBradesco($row) {
        set_time_limit(0);
        
        $arquivo = $row['DSPATH'][0];
        
        $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory()."/");
        
        if (!$file->isFile()) {
            throw new \Exception("Falha ao baixar o arquivo.");
        }
        
        $arquivo = file($file->getPathname());
        
        $data = array();

        foreach($arquivo as $key => $linha) {
            if($key == 0 && substr($linha, 2, 7) != 'RETORNO') {
                throw new \Exception("O arquivo não é de retorno.");
                break;
            } else if($key != 0 && $key != count($arquivo)-1) {
                $agencia = substr($linha, 63, 4);
                $codigo = substr($linha, 67, 4);
                $conta = substr($linha, 72, 7);
                $digitoConta = substr($linha, 79, 1);
                $nmvinculo = substr($linha, 82, 38);
                $erros = substr($linha, 150, 2);

                $conta .= '-' . $digitoConta;

                $data[] = $this->montarDadoBradesco($agencia, $codigo, $conta, $nmvinculo, $erros);
            }
        }
        
        $data['sucesso'] = 'Processo finalizado com sucesso!';
        return $data;
        
    }

    public function montarDadoBradesco($agencia, $codigo, $conta, $nmvinculo, $erros) {
        // Divide cada caracter pra falara qual é o erro
        // Cada 'S' em posição diferente resulta em um erro diferente
        // S na posição 1 = Erro na Agência
        // S na posição 2 = Erro na Conta
        $mensagemErro = '';
        foreach(str_split($erros) as $key => $erro) {
            if($erro != ' ') {
                if($key == 0) {
                    $mensagemErro = "Erro na Agência";
                } else if($key == 1) {
                    if(empty($mensagemErro)) {
                        $mensagemErro = "Erro na Conta";
                    } else {
                        $mensagemErro .= " e Conta";
                    }
                }
            }
        }

        if(empty($mensagemErro)) {
            $mensagemErro = 'Sem erros';
        }
        
        $dado = array(
            'AGENCIA' => $agencia,
            'CODIGO' => $codigo,
            'CONTA' => $conta,
            'NMVINCULO' => trim($nmvinculo),
            'ERRO' => $mensagemErro
        );

        return $dado;
    }
}