<?php

namespace HCM\Geral\Controllers;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\CboService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO\Response\Error;

class CboController extends Crud
{
    protected $dataSourceName = 'gpe_cbo';
    protected $entityManager;
    protected $hcmParameters;
    protected $parameterBag;
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    protected $cboService;

    public function __construct(Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, CboService $cboService)
    {
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->cboService = $cboService;

        parent::__construct($dataSourceManager);
    }

    public function getCbo(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setCadastroCbo(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->cboService->setResponseObject($response);

            foreach ($request->getDataSet()->getRows() as $row) {
                $this->cboService->verificaNrCboDessaRow($row);
            }

            $cadastroCboPossuiErros = !empty($response->getMessages()) && $response->getMessages()[0]->getType() == "E";
            if ($cadastroCboPossuiErros) {
                return;
            }

            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteCadastroCboController(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}