<?php

namespace HCM\Geral\Entity;

use HCM\Util\TkEntity;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use HCM\FPA\Entity\FpacalculofolhaEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpaTpvincprocessoEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\Util\FPA;
use HCM\Util\TkDI;

class GpevinculomEntity extends TkEntity
{   
    const ORDEM_ALFABETICA = 'ALFABETICA';
    const ORDEM_MATRICULA = 'MATRICULA';
    const ORDEM_DATA_ADMISSAO = 'DATA_ADMISSAO';
    const ORDEM_DATA_RESCISAO = 'DATA_RESCISAO';
    const ORDEM_ESTRUTURA = 'ESTRUTURA';
    const ORDEM_LISTA_SINDICATOS = 'SINDICATOS';
    const ORDEM_LISTA_VINCULOS = 'LISTA_VINCULOS';

    const ESTAGIARIO = 2;

    private $_admitidoPeriodo;
    private $_demitidoPeriodo;
    private $_admitidoMesAnterior;
    private $_demitidoAnteriormente;
    private $_demitidoPosteriormente;
    private $_ferias;
    private $movimentacoes;
    private $_historico;
    private $nrtipocalculo;
    private $tp_processamento;
    
    public function __construct($entityManager, $hcmParameters){
        
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        parent::__construct(null,$entityManager);
    }
    
    
    private static function _ordenaQuery($ordem) {
        
        switch ($ordem) {
            case self::ORDEM_ALFABETICA:
                $nomeColuna = 'vm.nmvinculom';
                break;
            case self::ORDEM_DATA_ADMISSAO:
                $nomeColuna = 'vm.dtadmissaovinc';
                break;
            case self::ORDEM_DATA_RESCISAO:
                $nomeColuna = 'vm.dtrescisaovinc';
                break;
            case self::ORDEM_ESTRUTURA:
                $nomeColuna = 'vh.nrestrutlegal';
                break;
            case self::ORDEM_LISTA_SINDICATOS:
                $nomeColuna = 'vh.nrestrutsind';
                break;
            case self::ORDEM_MATRICULA:
                $nomeColuna = 'vm.cdmatricula';
                break;
            case self::ORDEM_LISTA_VINCULOS:
                $nomeColuna = 'vm.nrvinculom';
                break;
            case 'LISTA_NRESTRUTSIND':
                $nomeColuna = 'vm.nmvinculom';
                break;
            default:
                $nomeColuna = null;
        }
        
        $nomeColuna = 'vm.nmvinculom';

        return $nomeColuna;
    }

    public function retornaVinculos($nrorg, $mes_competencia, $ds_inicio, $ds_fim, $dsOrdem = null) {
        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim = trim(strtoupper($ds_fim)); 

        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mes_competencia));

        $condProcesso = 'GPE_VINCULOM.DTADMISSAOVINC <=  \''.$dtUltDiaMes.'\'';

        $condIntervalo = 'TRIM(UPPER(GPE_VINCULOM.NMVINCULOM)) BETWEEN :p_ds_inicio AND \''.$ds_fim.'\'';

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }

        $resultado = Gpevinculom::retornaVinculos($nrorg, $mes_competencia, $condProcesso, $condIntervalo, $ordenacao);
        return $resultado;
    }
    
    public function projetaVinculoFimAno($gpe_vinculoh, $mes_competenc) {
        $mes_competencia_ano_seguinte = $mes_competenc;
        $num_mes = $mes_competencia_ano_seguinte->format('m');
        
        if($num_mes == 12) $mes_competencia_ano_seguinte = DateUtil::adicionaIntervalo(DateUtil::setMonth($mes_competenc, 1), 1, DateUtil::ANO);

        $vinculoProjetado = $this->projetaVinculo($gpe_vinculoh,$mes_competencia_ano_seguinte,$num_mes);

        if ($vinculoProjetado->getIdcontribuisind() == GpevinculohEntity::IDCONTRIBUISIND_DESCONTOU) $vinculoProjetado->setIdcontribuisind(GpevinculohEntity::IDCONTRIBUISIND_SEMPRE);
        $this->entityManager->persist($vinculoProjetado);
        $this->entityManager->flush();

        unset($mes_competencia_ano_seguinte);
        unset($vinculoProjetado);
    }
    
    public function projetaVinculo($gpe_vinculoh,$mes_competencia_projetado,$num_mes) {
        $mes_competencia_atual = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());	
        
        if ($mes_competencia_projetado->format('d/m/Y') == $mes_competencia_atual->format('d/m/Y') && $num_mes == 12) 
            return $this;
        else {
            $vinculoUltimoHistorico = TkDI::getContainer()->vinculoh->retornaVinculoHistoricoMesCompetencia($this->hcmParameters->getNrorgTrabalho(), $gpe_vinculoh->getNrvinculom(), $mes_competencia_projetado);
            
            if ($vinculoUltimoHistorico->getDtmescompetenc()->format('d/m/Y') == $mes_competencia_projetado->format('d/m/Y')) //Se tem histórico de Janeiro, deve apenas sobrescrever o histórico em cima de si mesmo.
                $retorno = $vinculoUltimoHistorico; 
            else{
                $vinculoProjetado = clone $gpe_vinculoh;
                $vinculoProjetado->setNrvinculoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOH', 1, 12, true));
                $vinculoProjetado->setDtmescompetenc($mes_competencia_projetado);
                $vinculoProjetado->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $vinculoProjetado->setCdoperultatu($this->hcmParameters->getCdoperador());
                $vinculoProjetado->setNrorg($this->hcmParameters->getNrorgTrabalho());
                $vinculoProjetado->setDtinclusao(DateUtil::getDataDeString($this->hcmParameters->getDataAtual(), DateUtil::FORMATO_BRASILEIRO));
                $vinculoProjetado->setDtultatu(DateUtil::getDataDeString($this->hcmParameters->getDataAtual(), DateUtil::FORMATO_BRASILEIRO));
                
                $retorno = $vinculoProjetado;
            }

            return $retorno;
        }
    }
    
    public function verificaNumeroDeVinculo (){
    $vinculo = $this->retornaVinculosAtivosMesCompetencia($nrorg, $mesCompetencia, null, null, null, false, null, true, false, null, null, null,  null, intval($pis));
    }
    public function retornaVinculosAtivosMesCompetencia(
        $nrOrg, $mesCompetencia, $ds_inicio, $ds_fim, $dsOrdem, $rescindidos = false, $nrEstrutura = null,
        $verificaConfidencial = true, $paramCaged = false, $nrTipoVinculom = null, $nrVinculoEmpreg = null,
        $nrEstrutSind = null, $nrEvento = null, $pis = false, $dsvincnaocontempl = null, $ocupacao = null
    ) {
        $params = array();
        $gpe_pessoah = Repositories::GPE_PESSOAH;
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        $fpaItemLoteVinc = Repositories::FPA_ITEMLOTEVINC;
        $hierarquiaOrg = Repositories::HIERARQUIAORG;
        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mesCompetencia));
        $condIntervalo = '';
        $condEstrutura = '';
        if ($paramCaged === true) {
            $condProcesso = 'EXISTS (SELECT 1
                                      FROM $gpeMovimentacao mo
                                     WHERE mo.nrvinculom        = vm.nrvinculom
                                       AND mo.nrorg             = vm.nrorg
                                       AND TRUNC(mo.dtinimoviment) BETWEEN :p_mescompetencia AND :p_dtultdiames) AND vm.nrvinculoempreg <> 8';
            $params['p_mescompetencia'] = $mesCompetencia;
            $params['p_dtultdiames'] = $dtUltDiaMes;
        }else if($pis){
            $condProcesso = " vm.nrpessoa in  
                                (select ph.nrpessoa from $gpe_pessoah ph 
                                where ph.nrpispaseppes = :p_nrpis 
                                    and ph.nrorg = vm.nrorg 
                                    and ph.dtmescompetenc = (select max(phh.dtmescompetenc) from $gpe_pessoah phh
                                        where phh.nrorg = ph.nrorg 
                                        and phh.nrpessoa = ph.nrpessoa 
                                        and phh.dtmescompetenc <= :p_mescompetencia))
                            and (vm.dtrescisaovinc IS NULL
                            OR  vm.dtrescisaovinc >= :p_mescompetencia)
                            AND vm.dtadmissaovinc <=  :p_dtultdiames";
            
            $params['p_mescompetencia'] = $mesCompetencia;
            $params['p_dtultdiames'] = $dtUltDiaMes;
            $params['p_nrpis'] = $pis;
        }else {
            if ($nrEvento == null) {
                if ($rescindidos === true) {
                    $condProcesso = '(vm.dtrescisaovinc IS NULL
                                        OR vm.dtrescisaovinc >= :p_mescompetencia
                                      )
                                      AND vm.dtadmissaovinc <=  :p_dtultdiames';
                    $params['p_mescompetencia'] = $mesCompetencia;
                    $params['p_dtultdiames'] = $dtUltDiaMes;
                } else {
                    $condProcesso = '(vm.dtrescisaovinc IS NULL
                                        OR vm.dtrescisaovinc > :p_dtultdiames
                                      )
                                      AND ((vm.dtadmissaovinc <> vm.dtrescisaovinc) OR vm.dtrescisaovinc IS NULL)
                                      AND vm.dtadmissaovinc <=  :p_dtultdiames';
                    $params['p_dtultdiames'] = $dtUltDiaMes;
                }
            } else {
                $condProcesso = "EXISTS (SELECT 1
                                           FROM $fpaIteCalcFolha ic
                                     WHERE ic.nrvinculom = vm.nrvinculom
                                       AND ic.nrorg      = vm.nrorg
                                       AND ic.nreventom  = :p_nreventom
                                       AND ic.dtocorrencia BETWEEN :p_mescompetencia AND :p_dtultdiames)";
                $params['p_nreventom'] = $nrEvento;
                $params['p_mescompetencia'] = $mesCompetencia;
                $params['p_dtultdiames'] = $dtUltDiaMes;
            }
        }
        
        if($this->nrtipocalculo == 14 && $this->tp_processamento == 'CALCULO_APOLO_GRAVAR_NP')
        {
            $calculoFolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                $nrOrg, 1, 1, 1, $mesCompetencia, $mesCompetencia
            )[0];
            
            $nrcalculofolha = $calculoFolha->getNrcalculofolha();
            if($calculoFolha){
            
            $condProcesso .= " AND NOT EXISTS (SELECT 1
                                       FROM $fpaIteCalcFolha ic
                                 WHERE ic.nrvinculom = vm.nrvinculom
                                   AND ic.nrorg      = vm.nrorg
                                   AND ic.nreventom  in (3280,3281,3282,3290,3292,3320,3330,3360,3370,3400,3401,3410,3440,3450,3480,3490,4020,4030,4040,4050,4060,4070,3270,3272,3275,3310,3350)
                                   AND ic.nrcalculofolha = :p_nrcalculofolha) ";
                                   
            $params['p_nrcalculofolha'] = $nrcalculofolha;
                
            }
        }
        
        if($ds_inicio){ $params['p_ds_inicio'] = $ds_inicio;}
        if($ds_fim){ $params['p_ds_fim'] = $ds_fim;}
        
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'TRIM(UPPER(vm.nmvinculom)) BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUTURA':
                if ($paramCaged === false) {
                    $condIntervalo  = ' vh.nrestrutlegal = :p_ds_inicio';
                    if($ds_fim)
                    $condIntervalo .= " AND vh.nrestrutgeren = :p_ds_fim";
                } else {
                    $condIntervalo =
                        "(
                            (vh.nrestrutlegal = :p_ds_inicio AND (vh.nrestrutgeren = :p_ds_fim OR :p_ds_fim IS NULL))
                            OR
                            (EXISTS (SELECT *
                                       FROM $gpeMovimentacao mov
                                      WHERE mov.nrorg = vm.nrorg
                                        AND mov.nrvinculom = vm.nrvinculom
                                        AND mov.nrestruturam = :p_ds_inicio
                                        AND mov.nrtpmovtransfm = 9))
                         )";
                }
                break;
            case 'LISTA_VINCULO':
                unset($params['p_ds_fim']);
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN ($ds_inicio)";
                break;
            case 'LISTA_LOTE':
                unset($params['p_ds_fim']);
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN (select ltv.nrvinculom FROM $fpaItemLoteVinc ltv WHERE ltv.nrorg = $nrOrg AND ltv.nrlotevinc IN($ds_inicio))";
                break;
            case 'LISTA_ESTRUTURA':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= " AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= " AND vh.nrestrutsind IN ($ds_fim)";
                }
                break;
            case 'MATRICULA':
                $condIntervalo = 'vm.cdmatricula BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'vm.dtadmissaovinc BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUT_ALFABETICA_TPVINC':
                if($ds_inicio['DSINICIO']){
                    unset($params['p_ds_inicio']);
                    $condIntervalo = 'vh.nrestrutlegal in ('. implode(",", $ds_inicio['DSINICIO']) .') ';
                }
                if($ds_fim['DSFIM']){
                    unset($params['p_ds_fim']);
                    $condIntervalo .= 'and (vh.nrestrutgeren in ('. implode(",", $ds_fim['DSFIM']) .')) ';
                    
                }
                if($ds_inicio['DSINICIO2'] && $ds_fim['DSFIM2']){
                    $params['p_ds_inicio2'] = $ds_inicio['DSINICIO2'];
                    $params['p_ds_fim2'] = $ds_fim['DSFIM2'];
                    $condIntervalo .= 'and (trim(vm.nmvinculom) between :p_ds_inicio2 and :p_ds_fim2) ';
                    if(strlen($ds_fim['DSFIM']) == 0){
                        unset($params['p_ds_fim']);
                    }
                }
                if($ds_inicio['DSINICIO3']){
                    $dsInicio3 = implode(",", $ds_inicio['DSINICIO3']);
                    $condIntervalo .= 'and vm.nrtipovinculom in ('.$dsInicio3.') ';
                }
                break;
            case 'TIPO_HIERARQUIA':
                if($ds_fim['DSFIM2']){
                    unset($params['p_ds_inicio']);
                    unset($params['p_ds_fim']);
                    $ds_fim['DSFIM2'] = is_array($ds_fim['DSFIM2']) ? implode(',',$ds_fim['DSFIM2']) : $ds_fim['DSFIM2'];
                    $condIntervalo .= ' (vh.nrestrutgeren in ('.$ds_fim['DSFIM2'].')) ';
                }else{
                    $ds_inicio['DSINICIO'] = is_array($ds_inicio['DSINICIO']) ? implode(',',$ds_inicio['DSINICIO']) : $ds_inicio['DSINICIO'];
                    $ds_inicio['DSINICIO2'] = is_array($ds_inicio['DSINICIO2']) ? implode(',',$ds_inicio['DSINICIO2']) : $ds_inicio['DSINICIO2'];
                    $ds_fim['DSFIM2'] = is_array($ds_fim['DSFIM2']) ? implode(',',$ds_fim['DSFIM2']) : $ds_fim['DSFIM2'];
                    unset($params['p_ds_inicio']);
                    unset($params['p_ds_fim']);
                    $condIntervalo .= "EXISTS (SELECT 1 
                                                 FROM $hierarquiaOrg h 
                                                WHERE h.nrorg = vm.nrorg 
                                                  AND (h.nrestruturamrel  = vh.nrestrutgeren OR h.nrestruturamsup = vh.nrestrutlegal)   
                                                  AND h.nrestruturamsup  IN (".$ds_inicio['DSINICIO2'].") 
                                                  AND h.nrtipohierarqorg = ".$ds_inicio['DSINICIO']." 
                                                  AND h.dtinivigencia = ( SELECT MAX(h1.dtinivigencia) 
                                                                            FROM $hierarquiaOrg h1 
                                                                           WHERE h1.dtinivigencia <= :p_dtultdiames 
                                                                             AND h1.nrorg = h.nrorg 
                                                                             AND h1.nrestruturamrel = h.nrestruturamrel 
                                                                             AND h1.nrtipohierarqorg = h.nrtipohierarqorg))";
                }
                break;
            case 'LISTA_OCUPACAO':
                if($ds_inicio){
                    $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                    unset($params['p_ds_inicio']);
                    $condIntervalo .= ' (vh.nrcargo in ('.$ds_inicio.')) ';
                }
                break;
            case 'LISTA_TOMADOR':
                if($ds_inicio) {
                    unset($params['p_ds_inicio']);
                    $ds_inicio = is_array($ds_inicio) ? implode($ds_inicio, ",") : $ds_inicio;
                    $condIntervalo .= "\r\n EXISTS (SELECT 1 
    	                FROM $hierarquiaOrg h 
    				   WHERE h.nrorg = $nrOrg
                         AND h.nrestruturamsup IN ($ds_inicio)
                         AND h.nrestruturamrel = vh.nrestrutgeren) 	";
                }
                if($ds_fim){
                    unset($params['p_ds_fim']);
                    $ds_fim = is_array($ds_fim) ? implode($ds_fim, ",") : $ds_fim;
                    $condIntervalo .= "\r\n  AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
        }
        
        if ($nrEstrutura !== null){
            $condEstrutura = 'vh.nrestrutlegal = :p_nrestrutura';
            $params['p_nrestrutura'] = $nrEstrutura;
        } else {
            $condEstrutura = null;
        }

        if (!empty($nrEstrutSind)){
            if ($condEstrutura !== null) {		
				$condEstrutura .= ' AND vh.nrestrutsind IN ('.implode($nrEstrutSind, ",").')';
			} else {
				$condEstrutura = 'vh.nrestrutsind IN ('.implode($nrEstrutSind, ",").')';
			}
        } 			
		
        if (!empty($nrTipoVinculom)){
            if ($condEstrutura !== null) {		
				$condEstrutura .= ' AND vm.nrtipovinculom = :p_nrtipovinculom';
			} else {
				$condEstrutura = 'vm.nrtipovinculom = :p_nrtipovinculom';
			}
            $params['p_nrtipovinculom'] = $nrTipoVinculom;
        } 			
		
        if (!empty($nrVinculoEmpreg)){
            if ($condEstrutura !== null) {		
				$condEstrutura .= ' AND vm.nrvinculoempreg = :p_nrvinculoempreg';
			} else {
				$condEstrutura = 'vm.nrvinculoempreg = :p_nrvinculoempreg';
			}
            $params['p_nrvinculoempreg'] = $nrVinculoEmpreg;
        } 					
		
		if(!empty($dsvincnaocontempl)){
		    if ($condEstrutura !== null) {		
				$condEstrutura .= ' AND vm.nrvinculom NOT IN ('.$dsvincnaocontempl.')';
			} else {
				$condEstrutura = 'vm.nrvinculom NOT IN ('.$dsvincnaocontempl.')';
			}
		}

		if(!empty($ocupacao)){
		    if ($condEstrutura !== null) {		
				$condEstrutura .= ' AND vh.nrcargo IN ('.implode($ocupacao, ",").')';
			} else {
				$condEstrutura = 'vh.nrcargo IN ('.implode($ocupacao, ",").')';
			}
		}
        
        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }

        $resultado = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura, $verificaConfidencial, $nrTipoVinculom, $nrVinculoEmpreg, null, $params);
        
        unset($dtUltDiaMes);
        return $resultado;
    }

    public function retornaVinculosAtivosComMovimentacaoMesCompetencia(
        $nrOrg, $mesCompetencia, $ds_inicio, $ds_fim, $dsOrdem, $nrTipotransfer, $nrTpmovtransfm, $nrTipoestrutura, $nrEstrutura,
        $nrTipoVinculom = null, $nrVinculoEmpreg = null, $verificaConfidencial = true
    ) {
        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim = trim(strtoupper($ds_fim));

        $dtUltDiaMes = new TkDateTime($mesCompetencia);
        $dtUltDiaMes->setDay(1)->addMonth(1)->subDay(1);

        $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NULL OR GPE_VINCULOM.DTRESCISAOVINC > \''.$dtUltDiaMes.'\')
						AND ((GPE_VINCULOM.DTADMISSAOVINC <> GPE_VINCULOM.DTRESCISAOVINC) OR GPE_VINCULOM.DTRESCISAOVINC IS NULL)
                        AND GPE_VINCULOM.DTADMISSAOVINC <= \''.$dtUltDiaMes.'\'';
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'TRIM(GPE_VINCULOM.NMVINCULOM) BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'ESTRUTURA':
                $condIntervalo = 'GPE_VINCULOH.NRESTRUTLEGAL = ' . $ds_inicio . ' ';
                $condIntervalo .= "AND (GPE_VINCULOH.NRESTRUTGEREN = '$ds_fim' OR '$ds_fim' IS NULL)";
                break;
            case 'MATRICULA':
                $condIntervalo = 'GPE_VINCULOM.CDMATRICULA BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'LISTA_VINCULOS':
                if ($ds_inicio === 'N') {
                    $condIntervalo = "'$ds_inicio' <> 'N'";
                }
                elseif ($ds_inicio === 'A') {
                    $condIntervalo = "'$ds_inicio' = 'A'";
                }
                else {
                    $condIntervalo = "GPE_VINCULOM.NRVINCULOM IN ($ds_inicio)";
                }
                break;
        }

        $condMovimentacao =
            "(EXISTS (SELECT *
                        FROM GPE_MOVIMENTACAO
                       WHERE NRORG = GPE_VINCULOM.NRORG
                         AND NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                         AND NRTIPOTRANSFER = $nrTipotransfer
                         AND NRTPMOVTRANSFM = $nrTpmovtransfm
                         AND NRTIPOESTRUTURA = $nrTipoestrutura
                         AND NRESTRUTURAM = $nrEstrutura
                         AND DTINIMOVIMENT BETWEEN '$mesCompetencia' AND '$dtUltDiaMes')
             )";

        if ($nrTipoVinculom !== null){
            if ($condMovimentacao !== null) {
                $condMovimentacao .= 'AND GPE_VINCULOM.NRTIPOVINCULOM = '.$nrTipoVinculom;
            } else {
                $condMovimentacao = 'GPE_VINCULOM.NRTIPOVINCULOM = '.$nrTipoVinculom;
            }
        }

        if ($nrVinculoEmpreg !== null){
            if ($condMovimentacao !== null) {
                $condMovimentacao .= 'AND GPE_VINCULOM.NRVINCULOEMPREG = '.$nrVinculoEmpreg;
            } else {
                $condMovimentacao = 'GPE_VINCULOM.NRVINCULOEMPREG = '.$nrVinculoEmpreg;
            }
        }

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }

        $resultado = Gpevinculom::retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condMovimentacao, $verificaConfidencial, $nrTipoVinculom, $nrVinculoEmpreg, null);

        unset($dtUltDiaMes);
        return $resultado;
    }

    public function getVinculosAtivosParaEvento($nrOrg, $mesCompetencia, $ds_inicio, $ds_fim,
                                                       $nreventom, $tpmovimento, $dsOrdem = null, $dtPagamento = null, $tpvinculo = null, $bloqueio = 'N' )
    {
        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mesCompetencia));
        $primeiroDiaAno = DateUtil::subtraiIntervalo(DateUtil::getPrimeiroDiaDoAno(DateUtil::getDataDeString($mesCompetencia)), 2, DateUtil::ANO);
        $params = array();
        $ocorrencia     = FPA::getOcorrenciaCalculo();
        $nrtpmodalidcal = FPA::getNrModalidadeCalculo();
        $params['nrorg'] = $nrOrg;
        $params['nreventom'] = $nreventom;
        $params['tpmovimento'] = $tpmovimento;
        $params['nrtpmodalidcal'] = $nrtpmodalidcal;
        $params['mescompetencia'] = $mesCompetencia->format('d/m/Y');
        $params['p_dtultdiames'] = $dtUltDiaMes->format('d/m/Y');
        $params['ocorrencia'] = $ocorrencia;
        $params['p_primeirodiaano'] = $primeiroDiaAno->format('d/m/Y');
        $params['p_ds_inicio'] = $ds_inicio;
        if($ds_fim){ $params['p_ds_fim'] = $ds_fim;}
        $params['p_bloqueio'] = $bloqueio;
        
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        $fpaCalculoFolha = Repositories::FPA_CALCULOFOLHA;
        $fpaItemLoteVinc = Repositories::FPA_ITEMLOTEVINC;
        $gpevinculom =  Repositories::GPE_VINCULOM;
        $gpevinculoh = Repositories::GPE_VINCULOH;
        $hierarquiaorg = Repositories::HIERARQUIAORG;
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'AND TRIM(UPPER(vm.nmvinculom)) BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUTURA':
                $condIntervalo  = 'AND vh.nrestrutlegal = :p_ds_inicio';
                if($ds_fim){
                    $condIntervalo .= " AND vh.nrestrutgeren = :p_ds_fim";
                }
                break;
            case 'LISTA_VINCULO':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "AND vm.nrvinculom IN ($ds_inicio)";
                break;
            case 'LISTA_LOTE':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "AND vm.nrvinculom IN (select ltv.nrvinculom FROM $fpaItemLoteVinc ltv WHERE ltv.nrorg = $nrOrg AND ltv.nrlotevinc IN($ds_inicio))";
                break;
            case 'LISTA_ESTRUTURA':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "AND vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= "AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = " AND vh.nrestrutlegal IN ($ds_inicio) ";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= " AND vh.nrestrutsind IN ($ds_fim) ";
                }
                break;
            case 'MATRICULA':
                $condIntervalo = 'AND vm.cdmatricula BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'AND vm.dtadmissaovinc BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUTURA_SIND':
                unset($params['p_ds_inicio']);
                $condIntervalo = "AND vh.nrestrutsind IN ($ds_inicio)";
                break;
            case 'LISTA_TOMADOR':
                if($ds_inicio) {
                    unset($params['p_ds_inicio']);
                    $ds_inicio = is_array($ds_inicio) ? implode($ds_inicio, ",") : $ds_inicio;
                    $condIntervalo .= "\r\n AND EXISTS (SELECT 1 
    	                FROM $hierarquiaorg h 
    				   WHERE h.nrorg = 3260
                         AND h.nrestruturamsup IN ($ds_inicio)
                         AND h.nrestruturamrel = vh.nrestrutgeren) 	";
                }
                if($ds_fim){
                    unset($params['p_ds_fim']);
                    $ds_fim = is_array($ds_fim) ? implode($ds_fim, ",") : $ds_fim;
                    $condIntervalo .= "\r\n  AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
            default:
                throw new \exception("Opção de ordenamento inválida ($this->ordem).");
        }

        if($tpvinculo){
            $condIntervalo .= "AND vm.nrtipovinculom IN ($tpvinculo) ";
        }
        
         if($dtPagamento){
            $dtPagamento = DateUtil::getDataDeString($dtPagamento);
            $condIntervalo .= " AND (itc.dtocorrencia = :dtpagamento OR :dtpagamento IS NULL)";
            $params['dtpagamento'] = $dtPagamento->format('d/m/Y');
        }

        $ordenacao = 'ORDER BY vm.nmvinculom';
        
        $sql = <<<SQL
SELECT vm, vh
FROM $gpevinculom vm
JOIN $gpevinculoh vh WITH
vh.dtmescompetenc =
    (SELECT MAX(vhh.dtmescompetenc)
        FROM $gpevinculoh vhh
        WHERE vhh.dtmescompetenc BETWEEN :p_primeirodiaano AND :p_dtultdiames
        AND vhh.nrorg = vh.nrorg
        AND vhh.nrvinculom = vh.nrvinculom
    )
AND vh.nrvinculom = vm.nrvinculom
AND vh.nrorg = vm.nrorg
AND vh.confidencial   = 'N'
JOIN $fpaIteCalcFolha itc
with vm.nrorg       = itc.nrorg
AND vm.nrvinculom = itc.nrvinculom
AND (:p_bloqueio = 'N' OR NOT EXISTS (SELECT 1 FROM $fpaIteCalcFolha itca WHERE itca.nrvinculom = vm.nrvinculom AND itca.nrorg = vm.nrorg AND itca.nreventom = '3037' AND itca.nrcalculofolha = itc.nrcalculofolha))
JOIN $fpaCalculoFolha cf
with cf.nrorg           = itc.nrorg
AND cf.nrcalculofolha = itc.nrcalculofolha
WHERE vm.nrorg = :nrorg
AND itc.nreventom      = :nreventom
AND itc.vreventofolha  <> 0
AND cf.nrtipomoviment = :tpmovimento
AND cf.nrtpmodalidcal = :nrtpmodalidcal
AND cf.dtocorrencia BETWEEN :mescompetencia AND :p_dtultdiames
AND cf.nrocorrecal = :ocorrencia
SQL;
        $sql = $sql.' '.$condIntervalo.' '.$ordenacao;

        $sqlParams = array('SQL' => $sql, 'PARAMS' =>$params);
        
        $result = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->_retornaVinculos($nrOrg, $sqlParams, $mesCompetencia);

        return $result;
    }

    public function retornaVinculosDemitidosMesCompetencia(
        $nrOrg, $mesCompetencia, $ds_inicio, $ds_fim, $dt_ini, $dt_fim, $dsOrdem,
        $nrEstrutura = null, $paramCaged = false
    ) {
        $params = array();
        $fpaItemLoteVinc = Repositories::FPA_ITEMLOTEVINC;
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        $ultimoDiaMesCompetencia = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mesCompetencia));
        $ds_inicio = is_string($ds_inicio) ? trim(strtoupper($ds_inicio)) : $ds_inicio;
        $ds_fim = is_string($ds_fim) ? trim(strtoupper($ds_fim)) : $ds_fim;
        
        if (($dt_ini === NULL) || ($dt_fim === NULL)) {
            $condProcesso = '(vm.dtrescisaovinc IS NOT NULL
                  		  AND (vm.dtrescisaovinc BETWEEN :p_mescompetencia AND :p_dtultdiames))
                         AND vm.dtadmissaovinc <= :p_dtultdiames';
                         $params['p_mescompetencia'] = $mesCompetencia;
        }else{
            $condProcesso = '(vm.dtrescisaovinc IS NOT NULL AND vm.dtrescisaovinc BETWEEN :dtini AND :dtfim
                          AND vm.dtadmissaovinc <=  :p_dtultdiames)';
                          $params['dtini'] = $dt_ini->format('d/m/Y');
                          $params['dtfim'] = $dt_fim->format('d/m/Y');
        }

        $condIntervalo = 'TRIM(vm.nmvinculom) BETWEEN :p_ds_inicio AND :p_ds_fim';
        
        $params['p_ds_inicio'] = $ds_inicio;
        if($ds_fim){ $params['p_ds_fim'] = $ds_fim;}
        
       switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'TRIM(UPPER(vm.nmvinculom)) BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUTURA':
                if ($paramCaged === false) {
                    $condIntervalo  = ' vh.nrestrutlegal = :p_ds_inicio';
                    if($ds_fim)
                    $condIntervalo .= " AND vh.nrestrutgeren = :p_ds_fim";
                } else {
                    $condIntervalo =
                        "(
                            (vh.nrestrutlegal = :p_ds_inicio AND (vh.nrestrutgeren = :p_ds_fim OR :p_ds_fim IS NULL))
                            OR
                            (EXISTS (SELECT mov
                                       FROM $gpeMovimentacao mov
                                      WHERE mov.nrorg = vm.nrorg
                                        AND mov.nrvinculom = vm.nrvinculom
                                        AND mov.nrestruturam = :p_ds_inicio
                                        AND mov.nrtpmovtransfm = 9))
                         )";
                }
                break;
            case 'LISTA_VINCULO':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN ($ds_inicio)";
                break;
            case 'LISTA_LOTE':
                unset($params['p_ds_fim']);
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN (select ltv.nrvinculom FROM $fpaItemLoteVinc ltv WHERE ltv.nrorg = $nrOrg AND ltv.nrlotevinc IN($ds_inicio))";
                break;
            case 'LISTA_ESTRUTURA':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= "AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= "AND vh.nrestrutsind IN ($ds_fim)";
                }
                break;
            case 'MATRICULA':
                $condIntervalo = 'vm.cdmatricula BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'vm.dtadmissaovinc BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
        }
        if ($nrEstrutura !== null){
            $condEstrutura = 'vh.nrestrutlegal = :p_nrestrutura';
            $params['p_nrestrutura'] = $nrEstrutura;
        } else {
            $condEstrutura = null;
        }
        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }
		
        $resultado = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura, null, null, null, null, $params);

        unset($dtUltDiaMes);

        return $resultado;
    }
	
    public function retornaVinculosDemitidosGeral(
        $nrOrg, $mesCompetencia, $ds_inicio, $ds_fim, $dt_ini, $dt_fim, $dsOrdem,
        $nrEstrutura = null, $paramCaged = false
    ) {
        $params = array();
        $fpaItemLoteVinc = Repositories::FPA_ITEMLOTEVINC;
        $ultimoDiaMesCompetencia = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mesCompetencia));
        
        if (($dt_ini === NULL) || ($dt_fim === NULL)) {
            $condProcesso = '(vm.dtrescisaovinc IS NOT NULL
                          AND vm.dtadmissaovinc <=  :p_dtultdiames)';
        }else{
            $condProcesso = '(vm.dtrescisaovinc IS NOT NULL AND vm.dtrescisaovinc BETWEEN :dtini AND :dtfim
                          AND vm.dtadmissaovinc <=  :p_dtultdiames)';
                          $params['dtini'] = substr($dt_ini, 0, 10);
                          $params['dtfim'] = substr($dt_fim, 0, 10);
        }

        $condIntervalo = 'TRIM(vm.nmvinculom) BETWEEN :p_ds_inicio AND :p_ds_fim';
        
        $params['p_ds_inicio'] = $ds_inicio;

        if($ds_fim){ $params['p_ds_fim'] = $ds_fim;}
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'TRIM(UPPER(vm.nmvinculom)) BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUTURA':
                if ($paramCaged === false) {
                    $condIntervalo  = ' vh.nrestrutlegal = :p_ds_inicio';
                    if($ds_fim)
                    $condIntervalo .= " AND vh.nrestrutgeren = :p_ds_fim";
                } else {
                    $condIntervalo =
                        "(
                            (vh.nrestrutlegal = :p_ds_inicio AND (vh.nrestrutgeren = :p_ds_fim OR :p_ds_fim IS NULL))
                            OR
                            (EXISTS (SELECT mov
                                       FROM $gpeMovimentacao mov
                                      WHERE mov.nrorg = vm.nrorg
                                        AND mov.nrvinculom = vm.nrvinculom
                                        AND mov.nrestruturam = :p_ds_inicio
                                        AND mov.nrtpmovtransfm = 9))
                         )";
                }
                break;
            case 'LISTA_VINCULO':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN ($ds_inicio)";
                break;
            case 'LISTA_LOTE':
                unset($params['p_ds_fim']);
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN (select ltv.nrvinculom FROM $fpaItemLoteVinc ltv WHERE ltv.nrorg = $nrOrg AND ltv.nrlotevinc IN($ds_inicio))";
                break;
            case 'LISTA_ESTRUTURA':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= "AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= "AND vh.nrestrutsind IN ($ds_fim)";
                }
                break;
            case 'MATRICULA':
                $condIntervalo = 'vm.cdmatricula BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'vm.dtadmissaovinc BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'DATA_RESCISAO':
                //$condIntervalo = 'vm.dtrescisaovinc BETWEEN :p_ds_inicio AND :p_ds_fim';
                $condIntervalo  = ' vh.nrestrutlegal = :p_ds_inicio';
                if($ds_fim)
                    $condIntervalo .= " AND vh.nrestrutgeren = :p_ds_fim";
                break;
        }
        if ($nrEstrutura !== null){
            $condEstrutura = 'vh.nrestrutlegal = '.$nrEstrutura;
            $params['p_nrestrutura'] = $nrEstrutura;
        } else {
            $condEstrutura = null;
        }
            

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }
        
        $desconsideraCompetencia = true;
        $resultado = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura, $desconsideraCompetencia, null, null, null, $params);
        unset($dtUltDiaMes);
        return $resultado;
    }

    public function retornaVinculosDemitidosAdmitidosMesCompetencia(
        $nrOrg, $mesCompetencia, $ds_inicio, $ds_fim, $dt_ini, $dt_fim, $dsOrdem,
        $nrEstrutura = null, $paramCaged = null
    ) {

        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim = trim(strtoupper($ds_fim));

        $ultimoDiaMesCompetencia = clone $mesCompetencia;
        $ultimoDiaMesCompetencia->setDay(1)->addMonth(1)->subDay(1);

        if (($dt_ini === NULL) || ($dt_fim === NULL)) {
            $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NOT NULL
                  		  AND (GPE_VINCULOM.DTRESCISAOVINC BETWEEN \''.$mesCompetencia.'\' AND \''.$ultimoDiaMesCompetencia.'\'))
                         OR GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$mesCompetencia.'\' AND \''.$ultimoDiaMesCompetencia.'\'';
        }else{
            $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NOT NULL
                              AND (GPE_VINCULOM.DTRESCISAOVINC BETWEEN \''.$dt_ini.'\' AND \''.$dt_fim.'\'))
                             OR GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$mesCompetencia.'\' AND \''.$ultimoDiaMesCompetencia.'\'';
        }

        $condIntervalo = '';

        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'TRIM(GPE_VINCULOM.NMVINCULOM) BETWEEN :p_ds_inicio AND \''.$ds_fim.'\'';
                break;
            case 'ESTRUTURA':
                $condIntervalo  = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$ds_inicio.' ';
                $condIntervalo .= "AND (GPE_VINCULOH.NRESTRUTGEREN = '$ds_fim' OR '$ds_fim' IS NULL)";
                break;
            case 'MATRICULA':
                $condIntervalo = 'GPE_VINCULOM.CDMATRICULA BETWEEN :p_ds_inicio AND \''.$ds_fim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condProcesso = null;
                $condIntervalo = 'GPE_VINCULOM.DTADMISSAOVINC BETWEEN :p_ds_inicio AND \''.$ds_fim.'\'';
                break;
        }

        if ($paramCaged !== null){
            $condIntervalo .= ' AND GPE_VINCULOM.IDSEGDESEMPVINC = \''.$paramCaged.'\'';
        }

        if ($nrEstrutura !== null){
            $condEstrutura = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$nrEstrutura;
        } else {
            $condEstrutura = null;
        }

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }

        $resultado = Gpevinculom::retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura);

        unset($dtUltDiaMes);

        return $resultado;
    }

    public function retornaDuracaoVinculo($dtAdmissaoVinc, $dtRescisaoVinc){
        $aux = new TkDateTime($dtRescisaoVinc, TkDateTime::FORMAT_DATE);
        $aux2 = new TkDateTime($dtAdmissaoVinc, TkDateTime::FORMAT_DATE);
        $periodo = $aux2->daysAfter($aux);
        return $periodo;
    }

    public function novoMestre(
        $nrpessoa, $nrorgTrab, $dtAdmissao, $dtRescisao, $matricula = null, $tipoVinculo = null,
        $nrvinculoempreg = null, $nrtpadmissao = null, $nmvinculom = null
    ){
        try{
            $vinculo = new self();

            $vinculo->setNrpessoa($nrpessoa);
            $vinculo->setNrorg($nrorgTrab);
            $vinculo->setDtadmissaovinc($dtAdmissao);
            $vinculo->setDtrescisaovinc($dtRescisao);
            $vinculo->setCdmatricula($matricula);
            $vinculo->setNrtipovinculom($tipoVinculo);
            $vinculo->setNrvinculoempreg($nrvinculoempreg);
            $vinculo->setNrtpadmissao($nrtpadmissao);
            $vinculo->setNmvinculom($nmvinculom);



            $vinculo->persist();
            // Verificar o retorno do numero do vinculo.
            $nrVinculo = $vinculo->getNrvinculom();
            unset($vinculo);
            return $nrVinculo;
        }
        catch (Exception $e){
            $e = new Exception('Erro ao criar um novo vinculo. Nao pode inserir um novo mestre.');
            throw $e;
        }

    }

    public function destroiMestre($nrvinculom, $nrorgTrab){
        try{
            $vinculom = Gpevinculom::retornaVinculo($nrvinculom, $nrorgTrab);
            if (is_object($vinculom) === true) {
                $this->addItem($vinculom, $nrvinculom);
                $this->removeItem($vinculom);
                $this->persist();

                return true;
            } else {
                return false;
            }
        }
        catch (Exception $e){
            throw new Exception('Nao foi possivel excluir o vinculo.');
        }

    }

    public function retornaVinculo($nrvinculom, $nrorg){
        return Gpevinculom::retornaVinculo($nrvinculom, $nrorg);
    }

    public function retornaVinculoPorMatriculaEstrutura($nrOrg, $matricula, $estrutura){
        $vinculo = Gpevinculom::retornaVinculoPorMatriculaEstrutura($nrOrg, $matricula, $estrutura);
        $vinculo = end($vinculo);
        return $vinculo;
    }

    public function retornaVinculosPorPessoa($nrOrg, $nrPessoa){
        return Gpevinculom::retornaVinculosPorPessoa($nrOrg, $nrPessoa);
    }

    public function retornaVinculoMestre($nrOrg){
        return end(Gpevinculom::retornaVinculoMestre($nrOrg));
    }

    public function retornaPK($nrOrg){
        $vinculo = $this->retornaVinculoMestre($nrOrg);
        return $vinculo->getNrvinculom();
    }

    public static function getVinculoMatricula($nrorg, $matricula) {
        $gpevinculom = self::getEntityByWhere('NRORG = ? AND CDMATRICULA = ?', array($nrorg, $matricula));
        return $gpevinculom;
    }

    public function insereVinculo($nrorg, $nrpessoa, $dtadmissao, $dtrescisao, $matricula, $tipovinculo,
                                  $nrvinculoempreg, $nrtpadmissao, $nmvinculom, $dtavisoprevio, $nrtpdemissao,
                                  $dtexamemedico)
    {
        $this->setNrorg($nrorg);
        $this->setNrpessoa($nrpessoa);
        $this->setNrtipovinculom($tipovinculo);
        $this->setNrvinculoempreg($nrvinculoempreg);
        $this->setNrtpadmissao($nrtpadmissao);
        $this->setNrtpdemissao($nrtpdemissao);
        $this->setNmvinculom($nmvinculom);
        $this->setCdmatricula($matricula);
        $this->setDtadmissaovinc($dtadmissao);
        $this->setDtrescisaovinc($dtrescisao);
        $this->setDtavisoprevio($dtavisoprevio);
        $this->setDtexamemedico($dtexamemedico);
    }

    public function getUltimoHistoricoCompetencia($dtmescompetencia) {
        $historico = Gpevinculoh::getUltimoHistoricoCompetencia(
            $this->getNrorg(), $this->getNrvinculom(), $dtmescompetencia
        );
        return $historico;
    }

    public function getFeriasNaoGozadas() {
        if (!isset($this->_ferias)) {
            $arrayFerias = Fpaferias::retornaItem($this->getNrvinculom(), $this->getNrorg());
            if ($arrayFerias) {
                $this->_ferias = new FpaferiasEntity();
            }
            foreach($arrayFerias as $ferias) {
                if (($ferias->getIdcontrolferias() !== FpaferiasEntity::IDCONTROLFERIAS_GOZADO_INTEGRAL) &&
                    ($ferias->getIdcontrolferias() !== FpaferiasEntity::IDCONTROLFERIAS_PERDIDO))
                {
                    $this->_ferias->addItem($ferias, $ferias->getNrferias());
                }
            }
        }
        return $this->_ferias;
    }

    public function getMovimentacoes($dtcompetencia) {
        $movimentacoes = Gpemovimentacao::retornaMovimentacoes(
            $this->getNrorg(), $this->getNrvinculom(), $dtcompetencia
        );
        return $movimentacoes;
    }

    public function getTipovinculo() {
        $tipovinculo = $this->getNrtipovinculom();
        return $tipovinculo;
    }

    public function getEscala() {
        $escala = GpeAlteescala::retornaAlteracaoEscalaPorVinculo(
            $this->getNrorg(), $this->getNrvinculom()
        );
        return $escala;
    }

    public function getOcupacao() {
        $ocupacao = GpeAlteocupacao::retornaAlteracaoOcupacaoVinculo(
            $this->getNrorg(), $this->getNrvinculom()
        );
        return $ocupacao;
    }

    public function getAlteracoesSalario() {
        $alteSalario = \Gpealtesalario::retornaAlteSalario($this->getNrorg(), $this->getNrvinculom());
        return $alteSalario;
    }

    public function getSituacoesFuncionais() {
        $alteSalario = \Gpealtesitufunc::getSituacoesVinculo($this->getNrorg(), $this->getNrvinculom());
        return $alteSalario;
    }

    public function getUltimaSituacaoFuncional($dtcompetencia) {
        $altesitufunc = \Gpealtesitufunc::getUltimaSituacaoPorDatainicio(
            $this->getNrorg(), $this->getNrvinculom(), $dtcompetencia
        );
        return $altesitufunc;
    }

    public function verificaLiquidoVinculos($nrorg, $mesCompetencia, $nrtipomoviment, $vinculos = null, $dsinicio = null, $dsfim = null, $dsordem = null,
                                            $rescindidos = true, $estagiarios = true)
    {
        $listaVinculos = array();

        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes($mesCompetencia);

        if (is_null($vinculos)) {
            $vinculos = $this->retornaVinculosAtivosMesCompetencia($nrorg, $mesCompetencia, $dsinicio, $dsfim, $dsordem, true);
        }

        $_calculoFolha = new FpacalculofolhaEntity($this->entityManager, $this->hcmParameters);
        $_itemCalculoFolha = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters);

        $listaNrCalculoMensal = $_calculoFolha->carregaListaNrCalculoFolhaTodasOcorrencias($nrorg, $nrtipomoviment, $mesCompetencia, 1);

        foreach ($vinculos as $vinculo) {

            if ((!$estagiarios) && ($vinculo->getNrtipovinculom() == self::ESTAGIARIO)) {
                continue;
            }

            $_tpVinculoProcesso = new FpaTpvincprocessoEntity($this->entityManager, $this->hcmParameters);
            $_tpVinculoProcesso->carregaTipoVinculoPorProcesso($nrorg, 0, $mesCompetencia);
            $listaProcessos = $_tpVinculoProcesso->getItem($vinculo->getNrtipovinculom());

            if (is_null($listaProcessos)) {
                continue;
            }

            if ($vinculo->getDtrescisaovinc() != null) $dtRescisaoVinc = DateUtil::getDataDeString($vinculo->getDtrescisaovinc());

            $cont_liquido = count($listaNrCalculoMensal);
            foreach ($listaNrCalculoMensal as $nrCalculo) {
                if (($vinculo->getDtrescisaovinc() == null) or ($dtRescisaoVinc > $dtUltDiaMes)) {
                    $liquido = $_itemCalculoFolha->retornaItensPorVinculoAndEvento($nrorg, $vinculo->getNrvinculom(), $nrCalculo, FpatipoassoceveEntity::ASSOC_LIQUIDO);
                    if ($liquido == 0) {
                        $cont_liquido--;
                    }
                } elseif ($rescindidos) { 
                    $liquido = $_itemCalculoFolha->retornaItensPorVinculoAndEvento($nrorg, $vinculo->getNrvinculom(), $nrCalculo, FpatipoassoceveEntity::ASSOC_LIQUIDO_RESCISAO);
                    if ($liquido == 0) {
                        $cont_liquido--;
                    }
                }
            }
            
            if ($cont_liquido == 0) {
                $listaVinculos[] = $vinculo;
            }
        }

        return empty($listaVinculos) ? true : $listaVinculos;
    }

    public function retornaVinculosRescisaoComplementarCompetencia(
        $nrOrg, $mesCompetencia, $ds_inicio, $ds_fim, $dsOrdem, $dt_ini, $dt_fim,
        $nrEstrutura = null, $verificaConfidencial = true
    ) {
        $params = array();

        $ultimoDiaMesCompetencia = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mesCompetencia));
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        $fpaItemLoteVinc = Repositories::FPA_ITEMLOTEVINC;
        $fpaCalculoFolha = Repositories::FPA_CALCULOFOLHA;
        $hierarquiaorg = Repositories::HIERARQUIAORG;
        $mesCompetencia = DateUtil::getDataDeString($mesCompetencia);
        
$condProcesso = <<<SQL
EXISTS (SELECT ic FROM $fpaIteCalcFolha ic
                                 INNER JOIN $fpaCalculoFolha cf
                                    WITH cf.nrorg = ic.nrorg
                                   AND cf.nrcalculofolha = ic.nrcalculofolha
                                WHERE cf.dtocorrencia = :p_mescompetencia
                                  AND ic.nrorg = :p_nrorg
                                  AND cf.nrtpmodalidcal = 1
                                  AND ic.nreventom = :p_nreventom
                                  AND ic.nrvinculom = vm.nrvinculom)
SQL;
        $params['p_mescompetencia'] = $mesCompetencia->format('d/m/Y');
        $params['p_nrorg'] = $nrOrg;
        $params['p_nreventom'] = FpatipoassoceveEntity::ASSOC_LIQUIDO_RESCISAO;

        if (($dt_ini === NULL) || ($dt_fim === NULL)) {
            $condProcesso .= ' AND (vm.dtrescisaovinc IS NOT NULL
                          AND vm.dtrescisaovinc < :p_mescompetencia
                          AND vm.dtadmissaovinc <= :p_dtultdiames)';
            $params['p_dtultdiames'] = $ultimoDiaMesCompetencia;
        }else{
            $condProcesso .= ' AND (vm.dtrescisaovinc IS NOT NULL
                          AND vm.dtrescisaovinc BETWEEN :p_dt_ini AND :p_dt_fim
                          AND vm.dtadmissaovinc <= :p_dtultdiames)';
            $params['p_dt_ini'] = $dt_ini;
            $params['p_dt_fim'] = $dt_fim;
            $params['p_dtultdiames'] = $ultimoDiaMesCompetencia;
        }
        
        $params['p_ds_inicio'] = $ds_inicio;
        if($ds_fim){
            $params['p_ds_fim'] = $ds_fim;
        }
        
        //Retirado em 26/05 por problema na geração da SEFIP //$condIntervalo = 'TRIM(vm.nmvinculom) BETWEEN :p_ds_inicio AND :p_ds_fim';
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'TRIM(vm.nmvinculom) BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUTURA':
                $condIntervalo  = 'vh.nrestrutlegal = :p_ds_inicio';
                if($ds_fim)
                $condIntervalo .= " AND vh.nrestrutgeren = :p_ds_fim";
                break;
            case 'MATRICULA':
                $condIntervalo = 'vm.cdmatricula BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'vm.dtadmissaovinc BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'LISTA_VINCULO':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN ($ds_inicio)";
                break;
            case 'LISTA_LOTE':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN (select ltv.nrvinculom FROM $fpaItemLoteVinc ltv WHERE ltv.nrorg = $nrOrg AND ltv.nrlotevinc IN($ds_inicio))";
                break;
            case 'LISTA_ESTRUTURA':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= " AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $condIntervalo = "vh.nrestrutlegal = $ds_inicio";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= "AND vh.nrestrutsind IN ($ds_fim)";
                }
                break;
            case 'LISTA_TOMADOR':
                if($ds_inicio) {
                    unset($params['p_ds_inicio']);
                    $ds_inicio = is_array($ds_inicio) ? implode($ds_inicio, ",") : $ds_inicio;
                    $condIntervalo .= "\r\n EXISTS (SELECT 1 
    	                FROM $hierarquiaorg h 
    				   WHERE h.nrorg = $nrOrg
                         AND h.nrestruturamsup IN ($ds_inicio)
                         AND h.nrestruturamrel = vh.nrestrutgeren) 	";
                }
                if($ds_fim){
                    unset($params['p_ds_fim']);
                    $ds_fim = is_array($ds_fim) ? implode($ds_fim, ",") : $ds_fim;
                    $condIntervalo .= "\r\n  AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
        }

        if ($nrEstrutura !== null){
            $condEstrutura = 'vh.nrestrutlegal = :p_nrestrutura';
            $params['p_nrestrutura'] = $nrEstrutura;
        } else {
            $condEstrutura = null;
        }

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }
        $resultado = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura, $verificaConfidencial, null, null, null, $params);

        unset($dtUltDiaMes);

        return $resultado;
    }

    public function retornaVinculosRescindidosComPlrCompetencia(
        $nrOrg, $mesCompetencia, $ds_inicio, $ds_fim, $dsOrdem, $dt_ini, $dt_fim,
        $nrEstrutura = null, $verificaConfidencial = true
    ) {
        $params = array();

        $ultimoDiaMesCompetencia = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mesCompetencia));
        $fpaItemLoteVinc = Repositories::FPA_ITEMLOTEVINC;
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        $fpaCalculoFolha = Repositories::FPA_CALCULOFOLHA;
        $mesCompetencia = DateUtil::getDataDeString($mesCompetencia);
        
$condProcesso = <<<SQL
EXISTS (SELECT ic FROM $fpaIteCalcFolha ic
                                 INNER JOIN $fpaCalculoFolha cf
                                    WITH cf.nrorg = ic.nrorg
                                   AND cf.nrcalculofolha = ic.nrcalculofolha
                                WHERE cf.dtocorrencia = :p_mescompetencia
                                  AND ic.nrorg = :p_nrorg
                                  AND cf.nrtpmodalidcal = 1
                                  AND cf.nrtipomoviment = :p_nrtipomoviment
                                  AND ic.nreventom = :p_nreventom
                                  AND ic.nrvinculom = vm.nrvinculom)
SQL;
        $params['p_mescompetencia'] = $mesCompetencia->format('d/m/Y');
        $params['p_nrorg'] = $nrOrg;
        $params['p_nreventom'] = FpatipoassoceveEntity::ASSOC_LIQUIDO;
        $params['p_nrtipomoviment'] = FpatipomovimentEntity::TP_MOV_PLR;

        if (($dt_ini === NULL) || ($dt_fim === NULL)) {
            $condProcesso .= ' AND (vm.dtrescisaovinc IS NOT NULL
                          AND vm.dtrescisaovinc < :p_mescompetencia
                          AND vm.dtadmissaovinc <= :p_dtultdiames)';
            $params['p_dtultdiames'] = $ultimoDiaMesCompetencia;
        }else{
            $condProcesso .= ' AND (vm.dtrescisaovinc IS NOT NULL
                          AND vm.dtrescisaovinc BETWEEN :p_dt_ini AND :p_dt_fim
                          AND vm.dtadmissaovinc <= :p_dtultdiames)';
            $params['p_dt_ini'] = $dt_ini;
            $params['p_dt_fim'] = $dt_fim;
            $params['p_dtultdiames'] = $ultimoDiaMesCompetencia;
        }
        
        $params['p_ds_inicio'] = $ds_inicio;
        if($ds_fim){ $params['p_ds_fim'] = $ds_fim;}
        
        $condIntervalo = 'TRIM(vm.nmvinculom) BETWEEN :p_ds_inicio AND :p_ds_fim';
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'TRIM(vm.nmvinculom) BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUTURA':
                $condIntervalo  = 'vh.nrestrutlegal = :p_ds_inicio';
                if($ds_fim)
                $condIntervalo .= " AND vh.nrestrutgeren = :p_ds_fim";
                break;
            case 'MATRICULA':
                $condIntervalo = 'vm.cdmatricula BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'vm.dtadmissaovinc BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            
            case 'LISTA_VINCULO':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN ($ds_inicio)";
                break;
            case 'LISTA_LOTE':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN (select ltv.nrvinculom FROM $fpaItemLoteVinc ltv WHERE ltv.nrorg = $nrOrg AND ltv.nrlotevinc IN($ds_inicio))";
                break;
            case 'LISTA_ESTRUTURA':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= " AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $condIntervalo = "vh.nrestrutlegal = $ds_inicio";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= "AND vh.nrestrutsind IN ($ds_fim)";
                }
                break;
        }

        if ($nrEstrutura !== null){
            $condEstrutura = 'vh.nrestrutlegal = :p_nrestrutura';
            $params['p_nrestrutura'] = $nrEstrutura;
        } else {
            $condEstrutura = null;
        }

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }
        $resultado = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura, $verificaConfidencial, null, null, null, $params);

        unset($dtUltDiaMes);

        return $resultado;
    }

    public function retornaVinculosRescindidosComBeneficioCompetencia(
        $nrOrg, $mesCompetencia, $ds_inicio, $ds_fim, $dsOrdem, $dt_ini, $dt_fim,
        $nrEstrutura = null, $verificaConfidencial = true
    ) {
        $params = array();

        $ultimoDiaMesCompetencia = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mesCompetencia));
        $fpaItemLoteVinc = Repositories::FPA_ITEMLOTEVINC;
        $fpaIteCalcFolha = Repositories::FPA_ITECALCFOLHA;
        $fpaCalculoFolha = Repositories::FPA_CALCULOFOLHA;
        $mesCompetencia = DateUtil::getDataDeString($mesCompetencia);
        
$condProcesso = <<<SQL
EXISTS (SELECT ic FROM $fpaIteCalcFolha ic
                                 INNER JOIN $fpaCalculoFolha cf
                                    WITH cf.nrorg = ic.nrorg
                                   AND cf.nrcalculofolha = ic.nrcalculofolha
                                WHERE cf.dtocorrencia = :p_mescompetencia
                                  AND ic.nrorg = :p_nrorg
                                  AND cf.nrtpmodalidcal = 1
                                  AND cf.nrtipomoviment = :p_nrtipomoviment
                                  AND ic.nrvinculom = vm.nrvinculom)
SQL;
        $params['p_mescompetencia'] = $mesCompetencia->format('d/m/Y');
        $params['p_nrorg'] = $nrOrg;
        //$params['p_nreventom'] = FpatipoassoceveEntity::ASSOC_LIQUIDO;
        $params['p_nrtipomoviment'] = FpatipomovimentEntity::TP_MOV_BENEFICIO;

        if (($dt_ini === NULL) || ($dt_fim === NULL)) {
            $condProcesso .= ' AND (vm.dtrescisaovinc IS NOT NULL
                          AND vm.dtrescisaovinc < :p_mescompetencia
                          AND vm.dtadmissaovinc <= :p_dtultdiames)';
            $params['p_dtultdiames'] = $ultimoDiaMesCompetencia;
        }else{
            $condProcesso .= ' AND (vm.dtrescisaovinc IS NOT NULL
                          AND vm.dtrescisaovinc BETWEEN :p_dt_ini AND :p_dt_fim
                          AND vm.dtadmissaovinc <= :p_dtultdiames)';
            $params['p_dt_ini'] = $dt_ini;
            $params['p_dt_fim'] = $dt_fim;
            $params['p_dtultdiames'] = $ultimoDiaMesCompetencia;
        }
        
        $params['p_ds_inicio'] = $ds_inicio;
        if($ds_fim){ $params['p_ds_fim'] = $ds_fim;}
        
        $condIntervalo = 'TRIM(vm.nmvinculom) BETWEEN :p_ds_inicio AND :p_ds_fim';
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'TRIM(vm.nmvinculom) BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUTURA':
                $condIntervalo  = 'vh.nrestrutlegal = :p_ds_inicio';
                if($ds_fim)
                $condIntervalo .= " AND vh.nrestrutgeren = :p_ds_fim";
                break;
            case 'MATRICULA':
                $condIntervalo = 'vm.cdmatricula BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'vm.dtadmissaovinc BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'LISTA_VINCULO':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN ($ds_inicio)";
                break;
            case 'LISTA_LOTE':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN (select ltv.nrvinculom FROM $fpaItemLoteVinc ltv WHERE ltv.nrorg = $nrOrg AND ltv.nrlotevinc IN($ds_inicio))";
                break;
            case 'LISTA_ESTRUTURA':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= " AND vh.nrestrutgeren IN ($ds_fim)";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $condIntervalo = "vh.nrestrutlegal = $ds_inicio";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= "AND vh.nrestrutsind IN ($ds_fim)";
                }
                break;
        }

        if ($nrEstrutura !== null){
            $condEstrutura = 'vh.nrestrutlegal = :p_nrestrutura';
            $params['p_nrestrutura'] = $nrEstrutura;
        } else {
            $condEstrutura = null;
        }

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }
        $resultado = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura, $verificaConfidencial, null, null, null, $params);

        unset($dtUltDiaMes);

        return $resultado;
    }

    public function getDtgerultcnab($vinculo, $nrcalculofolha, $nreventom, $dtPagamento)
    {
        $params = array('nrorg' => $vinculo->getNrorg(),'nrcalculofolha' => $nrcalculofolha,'nreventom' => $nreventom,'nrvinculom' => $vinculo->getNrvinculom());
        if ($dtPagamento){
            $params['dtocorrencia'] = DateUtil::getDataDeString($dtPagamento.' 00:00:00', DateUtil::FORMATO_BRASILEIRO_DATAHORA);
        }
        $resultado = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy($params);
        return is_null($resultado) ? null : $resultado->getDtgerultcnab();
    }
    
    public function getDtgerultgrrf($vinculo, $nrcalculofolha, $nreventom, $dtPagamento)
    {
        $params = array('nrorg' => $vinculo->getNrorg(),'nrcalculofolha' => $nrcalculofolha,'nreventom' => $nreventom,'nrvinculom' => $vinculo->getNrvinculom());
        if ($dtPagamento){
            $params['dtocorrencia'] = DateUtil::getDataDeString($dtPagamento.' 00:00:00', DateUtil::FORMATO_BRASILEIRO_DATAHORA);
        }
        $resultado = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy($params);
        
        return is_null($resultado) ? null : $resultado->getDtgerultgrrf();
    }

    public function getVinculoempreg()
    {
        $vinculoempreg = Gpevinculoempreg::retornaVinculoEmpregaticio($this->getNrvinculoempreg());
        return $vinculoempreg;
    }

    public function getVinculosEstrutLegalDifereMovimentacao($nrorg, $dtmescompetenc, $nrtipotransflegal, $nrtipoestrutlegal, $dsordem, $dsinicio, $dsfim, $ativos=true, $rescindidos=true)
    {
        
        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($dtmescompetenc));
        
        switch ($dsordem){
            case 'ALFABETICA':
                $condIntervalo = ' AND TRIM(vm.nmvinculom) BETWEEN \''.$dsinicio.'\' AND \''.$dsfim.'\'';
                break;
            case 'ESTRUTURA':
                $condIntervalo  = ' and vh.nrestrutlegal = '.$dsinicio.' ';
                if($dsfim)
                $condIntervalo .= "and vh.nrestrutgeren = '$dsfim'";
                break;
            case 'MATRICULA':
                $condIntervalo = ' AND vm.cdmatricula BETWEEN \''.$dsinicio.'\' AND \''.$dsfim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = ' AND vm.dtadmissaovinc BETWEEN \''.$dsinicio.'\' AND \''.$dsfim.'\'';
                break;
            case 'LISTA_VINCULO':
                $dsinicio = is_array($dsinicio) ? implode(',',$dsinicio) : $dsinicio;
                $condIntervalo = "AND vm.nrvinculom IN ($dsinicio)";
                break;
            case 'LISTA_ESTRUTURA':
                $dsinicio = is_array($dsinicio) ? implode(',',$dsinicio) : $dsinicio;
                $condIntervalo = "AND vh.nrestrutlegal IN ($dsinicio)";
                if($dsfim){
                    $dsfim = is_array($dsfim) ? implode(',',$dsfim): $dsfim;
                    $condIntervalo .= "AND vh.nrestrutgeren IN ($dsfim)";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                $condIntervalo = "AND vh.nrestrutlegal = $dsinicio";
                if($dsfim){
                    $dsfim = is_array($dsfim) ? implode(',',$dsfim) : $dsfim;
                    $condIntervalo .= " AND vh.nrestrutsind IN ($dsfim)";
                }
                break;
        }

        if ($ativos === true) {
            if ($rescindidos === true) {
                $condAtivos = ' AND (vm.dtrescisaovinc IS NULL
                                    OR vm.dtrescisaovinc >= :dtmescompetenc
                                  )
                                  AND vm.dtadmissaovinc <=  :dtultdiames';
            } else {
                $condAtivos = ' AND (vm.dtrescisaovinc IS NULL
                                    OR vm.dtrescisaovinc > :dtultdiames
                                  )
                                  AND vm.dtadmissaovinc <=  :dtultdiames';
            }
        }

        $vinculos = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculosEstrutLegalDifereMovimentacao($nrorg, $dtmescompetenc, $nrtipotransflegal, $nrtipoestrutlegal, $condIntervalo, $condAtivos);
        return $vinculos;
    }

    public function getVinculosEstrutGerenDifereMovimentacao($nrorg, $dtmescompetenc, $nrtipotransflegal, $nrtipoestrutlegal, $dsordem, $dsinicio, $dsfim, $ativos=true, $rescindidos=true)
    {
        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($dtmescompetenc));

        switch ($dsordem){
            case 'ALFABETICA':
                $condIntervalo = ' AND TRIM(vm.nmvinculom) BETWEEN \''.$dsinicio.'\' AND \''.$dsfim.'\'';
                break;
            case 'ESTRUTURA':
                $condIntervalo  = ' and vh.nrestrutlegal = '.$dsinicio.' ';
                if($dsfim)
                $condIntervalo .= "and vh.nrestrutgeren = '$dsfim'";
                break;
            case 'MATRICULA':
                $condIntervalo = ' AND vm.cdmatricula BETWEEN \''.$dsinicio.'\' AND \''.$dsfim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = ' AND vm.dtadmissaovinc BETWEEN \''.$dsinicio.'\' AND \''.$dsfim.'\'';
                break;
            case 'LISTA_VINCULO':
                $dsinicio = is_array($dsinicio) ? implode(',',$dsinicio) : $dsinicio;
                $condIntervalo = "AND vm.nrvinculom IN ($dsinicio)";
                break;
            case 'LISTA_ESTRUTURA':
                $dsinicio = is_array($dsinicio) ? implode(',',$dsinicio) : $dsinicio;
                $condIntervalo = "AND vh.nrestrutlegal IN ($dsinicio)";
                if($dsfim){
                    $dsfim = is_array($dsfim) ? implode(',',$dsfim) : $dsfim;
                    $condIntervalo .= "AND vh.nrestrutgeren IN ($dsfim)";
                }
                break;
            case 'LISTA_NRESTRUTSIND':
                $condIntervalo = "AND vh.nrestrutlegal = $dsinicio";
                if($dsfim){
                    $dsfim = is_array($dsfim) ? implode(',',$dsfim) : $dsfim;
                    $condIntervalo .= "AND vh.nrestrutsind IN ($dsfim)";
                }
                break;
        }

        if ($ativos === true) {
            if ($rescindidos === true) {
                $condAtivos = ' AND (vm.dtrescisaovinc IS NULL
                                    OR vm.dtrescisaovinc >= :dtmescompetenc
                                  )
                                  AND vm.dtadmissaovinc <=  :dtultdiames';
            } else {
                $condAtivos = ' AND (vm.dtrescisaovinc IS NULL
                                    OR vm.dtrescisaovinc > :dtultdiames
                                  )
                                  AND vm.dtadmissaovinc <=  :dtultdiames';
            }
        }

        $vinculos = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculosEstrutGerenDifereMovimentacao($nrorg, $dtmescompetenc, $nrtipotransflegal, $nrtipoestrutlegal, $condIntervalo, $condAtivos);
        return $vinculos;
    }

    public function retornaVinculosPorTipoCalculo($nrTipoCalculo,$nrOrgTrabalho, $mesCompetencia, $dsInicio, $dsFim, $dt_inicio, $dt_fim, $dsOrdem, $nrEstruturaTrab, $dsvincnaocontempl = null, $feriasColetiva = false, $_idConsvincresc = "N", $tp_processamento = null){
 
        $this->nrtipocalculo = $nrTipoCalculo;
        $this->tp_processamento = $tp_processamento;
        
        $_idConsvincrescin = $_idConsvincresc == 'S' ? true : false;
        
        if ($nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO ||
            $nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR)  {

            if ($nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO) {
                $vinculos = $this->retornaVinculosDemitidosMesCompetencia(
                    $nrOrgTrabalho, $mesCompetencia, $dsInicio, $dsFim,
                    $dt_inicio, $dt_fim, $dsOrdem, $nrEstruturaTrab
                );
            };
            
            if ($nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR) {
                $vinculos = $this->retornaVinculosDemitidosGeral(
                    $nrOrgTrabalho, $mesCompetencia, $dsInicio, $dsFim,
                    $dt_inicio, $dt_fim, $dsOrdem, $nrEstruturaTrab
                );
            };
        } else {
            if ($nrTipoCalculo === FpatipocalculoEntity::CALC_ADIANT_PLR ||
                $nrTipoCalculo === FpatipocalculoEntity::CALC_PLR) {
                    
                $vinculos = $this->retornaVinculosAtivosMesCompetencia(
                    $nrOrgTrabalho, $mesCompetencia, $dsInicio, $dsFim,
                    $dsOrdem, false, $nrEstruturaTrab, true, false, null, null, null, FpatipoassoceveEntity::ASSOC_PLR
                );
            } else {
                if ($nrTipoCalculo === FpatipocalculoEntity::CALC_PROVISAO_CONTABIL or $nrTipoCalculo === FpatipocalculoEntity::CALC_PROVISAO_OTIMIZADA){
                    $vinculos = $this->retornaVinculosAtivosMesCompetencia(
                        $nrOrgTrabalho, $mesCompetencia, $dsInicio, $dsFim, $dsOrdem,
                        false, $nrEstruturaTrab, true, false, 1, null, null, null);
                }else if($nrTipoCalculo === FpatipocalculoEntity::CALC_FERIAS && $feriasColetiva) {
                    $vinculos = $this->retornaVinculosAtivosMesCompetencia(
                        $nrOrgTrabalho, $mesCompetencia, $dsInicio, $dsFim,  $dsOrdem,
                        false, $nrEstruturaTrab, true, false, null, null, null, null, false, $dsvincnaocontempl
                    );
                }else if($nrTipoCalculo === FpatipocalculoEntity::CALC_CONTABILIZACAO){
                    $vinculos = $this->retornaVinculosAtivosMesCompetencia(
                        $nrOrgTrabalho, $mesCompetencia, $dsInicio, $dsFim,  $dsOrdem,
                        true, $nrEstruturaTrab
                    );
                    $vinculos_resc = $this->retornaVinculosRescisaoComplementarCompetencia(
                        $nrOrgTrabalho, $mesCompetencia, $dsInicio, $dsFim,  $dsOrdem,
                        null, null, $nrEstruturaTrab
                    );
                    if ($vinculos_resc) {
                        $vinculos = array_merge($vinculos, $vinculos_resc);
                    }
                }else {
                    
                    $vinculos = $this->retornaVinculosAtivosMesCompetencia(
                        $nrOrgTrabalho, $mesCompetencia, $dsInicio, $dsFim,  $dsOrdem,
                        $_idConsvincrescin, $nrEstruturaTrab, true, false, null,  null,
                        null, null, false, null
                    );
                }
            }
        }
        return $vinculos;
    }

    public function retornaVincMestreHistorico($nrOrg, $nrVinculom, $dtCompetencia)
    {
        $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVincMestreHistorico($nrOrg,$nrVinculom,$dtCompetencia);
        return $vinculo;
    }

    public function retornaVinculosDirf(
        $nrOrg, $mesCompetencia, $ds_inicio, $ds_fim, $dsOrdem, $tdVinculos, $idVincProLabore
    ) {
        $params = array();
        $hierarquiaOrg = Repositories::HIERARQUIAORG;
        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($mesCompetencia));
        $condIntervalo = '';
        $condEstrutura = '1=1';
        $verificaConfidencial = true;
        $nrTipoVinculom = null;
        $nrVinculoEmpreg = null;
        $nrEstrutSind = null;
        
        if($ds_inicio){ $params['p_ds_inicio'] = $ds_inicio;}
        if($ds_fim){ $params['p_ds_fim'] = $ds_fim;}
        
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'TRIM(UPPER(vm.nmvinculom)) BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUTURA':
                $condIntervalo  = ' vh.nrestrutlegal = :p_ds_inicio';
                if($ds_fim)
                    $condIntervalo .= " AND vh.nrestrutgeren = :p_ds_fim";
                $condEstrutura = str_replace('vh','vhh',$condIntervalo);
                break;
            case 'LISTA_VINCULO':
                unset($params['p_ds_inicio']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vm.nrvinculom IN ($ds_inicio)";
                break;
            case 'LISTA_ESTRUTURA':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= " AND vh.nrestrutgeren IN ($ds_fim)";
                }
                $condEstrutura = str_replace('vh','vhh',$condIntervalo);
                break;
            case 'LISTA_NRESTRUTSIND':
                unset($params['p_ds_inicio']);
                unset($params['p_ds_fim']);
                $condIntervalo = "vh.nrestrutlegal IN ($ds_inicio)";
                if($ds_fim){
                    $ds_fim = is_array($ds_fim) ? implode(',',$ds_fim) : $ds_fim;
                    $condIntervalo .= " AND vh.nrestrutsind IN ($ds_fim)";
                }
                $condEstrutura = str_replace('vh','vhh',$condIntervalo);
                break;
            case 'MATRICULA':
                $condIntervalo = 'vm.cdmatricula BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'vm.dtadmissaovinc BETWEEN :p_ds_inicio AND :p_ds_fim';
                break;
            case 'ESTRUT_ALFABETICA_TPVINC':
                if($ds_inicio['DSINICIO']){
                    unset($params['p_ds_inicio']);
                    $condIntervalo = 'vh.nrestrutlegal in ('. implode(",", $ds_inicio['DSINICIO']) .') ';
                }
                if($ds_fim['DSFIM']){
                    unset($params['p_ds_fim']);
                    $condIntervalo .= 'and (vh.nrestrutgeren in ('. implode(",", $ds_fim['DSFIM']) .')) ';
                    
                }
                $condEstrutura = str_replace('vh','vhh',$condIntervalo);
                if($ds_inicio['DSINICIO2'] && $ds_fim['DSFIM2']){
                    $params['p_ds_inicio2'] = $ds_inicio['DSINICIO2'];
                    $params['p_ds_fim2'] = $ds_fim['DSFIM2'];
                    $condIntervalo .= 'and (trim(vm.nmvinculom) between :p_ds_inicio2 and :p_ds_fim2) ';
                    if(strlen($ds_fim['DSFIM']) == 0){
                        unset($params['p_ds_fim']);
                    }
                }
                if($ds_inicio['DSINICIO3']){
                    $dsInicio3 = implode(",", $ds_inicio['DSINICIO3']);
                    $condIntervalo .= 'and vm.nrtipovinculom in ('.$dsInicio3.') ';
                }
                break;
            case 'TIPO_HIERARQUIA':
                if($ds_fim['DSFIM2']){
                    unset($params['p_ds_inicio']);
                    unset($params['p_ds_fim']);
                    $ds_fim['DSFIM2'] = is_array($ds_fim['DSFIM2']) ? implode(',',$ds_fim['DSFIM2']) : $ds_fim['DSFIM2'];
                    $condIntervalo .= ' (vh.nrestrutgeren in ('.$ds_fim['DSFIM2'].')) ';
                    $condEstrutura = str_replace('vh','vhh',$condIntervalo);
                }else{
                    $ds_inicio['DSINICIO'] = is_array($ds_inicio['DSINICIO']) ? implode(',',$ds_inicio['DSINICIO']) : $ds_inicio['DSINICIO'];
                    $ds_inicio['DSINICIO2'] = is_array($ds_inicio['DSINICIO2']) ? implode(',',$ds_inicio['DSINICIO2']) : $ds_inicio['DSINICIO2'];
                    $ds_fim['DSFIM2'] = is_array($ds_fim['DSFIM2']) ? implode(',',$ds_fim['DSFIM2']) : $ds_fim['DSFIM2'];
                    unset($params['p_ds_inicio']);
                    unset($params['p_ds_fim']);
                    $condIntervalo .= "EXISTS (SELECT 1 
                                                 FROM $hierarquiaOrg h 
                                                WHERE h.nrorg = vm.nrorg 
                                                  AND (h.nrestruturamrel  = vh.nrestrutgeren OR h.nrestruturamsup = vh.nrestrutlegal)   
                                                  AND h.nrestruturamsup  IN (".$ds_inicio['DSINICIO2'].") 
                                                  AND h.nrtipohierarqorg = ".$ds_inicio['DSINICIO']." 
                                                  AND h.dtinivigencia = ( SELECT MAX(h1.dtinivigencia) 
                                                                            FROM $hierarquiaOrg h1 
                                                                           WHERE h1.dtinivigencia <= :p_dtultdiames 
                                                                             AND h1.nrorg = h.nrorg 
                                                                             AND h1.nrestruturamrel = h.nrestruturamrel 
                                                                             AND h1.nrtipohierarqorg = h.nrtipohierarqorg))";
                }
                break;
            case 'LISTA_OCUPACAO':
                if($ds_inicio){
                    $ds_inicio = is_array($ds_inicio) ? implode(',',$ds_inicio) : $ds_inicio;
                    unset($params['p_ds_inicio']);
                    $condIntervalo .= ' (vh.nrcargo in ('.$ds_inicio.')) ';
                }
                break;  
        }

        $condProcesso = '(vm.dtrescisaovinc IS NULL OR vm.dtrescisaovinc > :p_primeirodiaano)
                          AND ((vm.dtadmissaovinc <> vm.dtrescisaovinc) OR vm.dtrescisaovinc IS NULL)
                          AND vm.dtadmissaovinc <=  :p_dtultdiames';
        $params['p_dtultdiames'] = $dtUltDiaMes;
		
        /* Temporáriamente o campo de pro labore foi tirado da tela e sempre vai gerar todos tipos de vínculo
        if ($tdVinculos == 'S') {
            if ($idVincProLabore == 'N'){
                if ($condProcesso !== null) {		
                    $condProcesso .= ' AND vm.nrtipovinculom NOT IN (2,3,10)';
                } else {
                    $condProcesso = 'vm.nrtipovinculom NOT IN (2,3,10)';
                }
            }
        }*/

        /* $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        } */
        $ordenacao = 'ORDER BY vm.nmvinculom';

        $resultado = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculosDirf(
            $nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura, $verificaConfidencial, $nrTipoVinculom, $nrVinculoEmpreg, $nrEstrutSind, $params, true
        );

        unset($dtUltDiaMes);
        return $resultado;
    }
    
}