<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S1298Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getDadosS1298($parametros) {
        try {
            $dados = $this->getDadosPrincipais($parametros['NRESTRUTURAM'], $parametros['NRTABELA']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosPrincipais($nrestruturam, $nrtabela) {
        $sql = <<<SQL
SELECT DISTINCT EMH.NRESTRUTURAM CHAVE,
       /*ideEvento*/
       DECODE(CF.NRTIPOMOVIMENT,1,1,10,1,19,1,4,2) as indApuracao,
       TO_CHAR(CF.DTOCORRENCIA, DECODE(CF.NRTIPOMOVIMENT,4,'YYYY','YYYY-MM')) as perApur,
       /*ideEmpregador*/
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC
  FROM ESTRUTURAH EMH
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Movimento*/
  LEFT JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = EMH.NRORG
                               AND CF.NRTIPOMOVIMENT = :P_NRTABELA
                               AND CF.DTOCORRENCIA = :P_DTMESCOMPETENC
                               AND CF.NRTPMODALIDCAL = 1
                               AND CF.NROCORRECAL = 1
 WHERE EMH.NRORG = :P_NRORG
   AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
   AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                AND ESTRUTURAH.NRORG = EMH.NRORG)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRTABELA" => $nrtabela,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

}