<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use Geral\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use \HCM\Geral\Services\VinculoServices;
use HCM\Geral\Entity\HierarquiaorgEntity;


class HierarquiaGridService {

    protected $entityManager;
    protected $hcmParameters;
    protected $hierarquiaGridRepository;
    protected $vinculoServices;

    private $informacoesParaValidacao;

    public const TIPO_ESTRUT_TOMADOR = 7;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, \HCM\Geral\Repository\HierarquiaGridRepository $hierarquiaGridRepository,VinculoServices $vinculoServices) {
        $this->entityManager            = $entityManager;
        $this->hcmParameters            = $hcmParameters;
        $this->hierarquiaGridRepository = $hierarquiaGridRepository;
        $this->vinculoServices          = $vinculoServices;
        $this->hierarquiaOrg            = new HierarquiaorgEntity($entityManager,$hcmParameters);
    }
    
    
    /** CADASTRO AUTOMÁTICO HIERARQUIA ESTRUTURA LEGAL/GERENCIAL **/
    
    /**
     * Função principal
     */
    public function setCadAutomaticoLegalGeren() {
        $quebraLinha    = "<br>";
        $nrorg          = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao    = $this->hcmParameters->getNrorgPadrao();
        $mesCompetencia = $this->hcmParameters->getDtCompetencia();
        
        /*Adiquiri o tipo de estruturas legal e gerencial dos parâmetros da organização*/
        $arrTipos = $this->getTiposLegalGeren($nrorg);

        /*Adiquiri o número do tipo de hierarquia. Caso esta não existe será criada neste ponto.*/
        $tipoHierarquia = $this->getTipoHierarquiaLegalGeren($nrorg, $nrorgPadrao, $arrTipos['nrtpestrutlegal'], $arrTipos['nrtpestrutgeren']);
        
        unset($arrTipos);
        
        //Lista do vínculos ativos
        $arrayVinculos = $this->vinculoServices->retornaVinculosAtivosCompetencia($nrorg, $mesCompetencia, null, null, null);
        
        //Monta lista de hierarquias
        $listaHierarquias = $this->montaListaHierarquia($arrayVinculos);
        
        //Insere as hierarquias
        $contador = $this->insereHierarquias($nrorg, $mesCompetencia, $tipoHierarquia['nrtipohierarqorg'], $tipoHierarquia['nmtipohierarqorg'], $listaHierarquias['HIERARQUIAS']);
        
        $mensagem = "Hierarquia " . $tipoHierarquia['nmtipohierarqorg'] . "." . $quebraLinha;
        
        if(count($listaHierarquias['semEstrutura']) > 0) {
            $mensagem .= $quebraLinha . "Vinculo(s) sem estrutura:" . $quebraLinha;
            foreach($listaHierarquias['semEstrutura'] as $msg) {
                $mensagem .= $msg . $quebraLinha;
            }
        }
        
        $mensagem .= "Registros inseridos: " . $contador['inseridos'] . $quebraLinha .
                     "Registros já existentes: " . $contador['naoinseridos'];

        return $mensagem;
    }
    
    /**
     * Adiquiri os tipos de estruturas legal e gerencial
     */
    private function getTiposLegalGeren($nrorg) {

        $params = array('nrorg' => $nrorg);
        $objParamorg = $this->entityManager->getRepository(Repositories::GPE_PARAMORG)->findOneBy($params);
        
        if(empty($objParamorg))
            throw new \Exception("Não há parâmetros para a organização número $nrorg.");
        
        $nrtpestrutlegal = $objParamorg->getNrtpestrutlegal();
        $nrtpestrutgeren = $objParamorg->getNrtpestrutgeren();
        
        if(empty($nrtpestrutlegal) || empty($nrtpestrutgeren))
            throw new \Exception("Não há um tipo de estrutura legal e/ou gerencial cadastrada no parâmetros da organização.");
        
        unset($paramOrg, $params);
        
        return array('nrtpestrutlegal' => $nrtpestrutlegal, 'nrtpestrutgeren' => $nrtpestrutgeren);
        
    }
    
    /**
     * Adiquiri o número do tipo de hierarquia para um relacionamento LEGAL / GERENCIAL.
     * Caso esta não existe relacionamento será criada um tipo novo de hierarquia e seu número será retornado.
     */
    private function getTipoHierarquiaLegalGeren($nrorg, $nrorgPadrao, $nrtipoestrutlegal, $nrtipoestrutgeren) {
        
        $params = array('nrorg'           => array($nrorg, $nrorgPadrao),
                        'nrtipoestrutsup' => $nrtipoestrutlegal,
                        'nrtipoestrutrel' => $nrtipoestrutgeren );
        $objTipohierarqorg = $this->entityManager->getRepository(Repositories::TIPOHIERARQORG)->findOneBy($params);
        
        unset($params);
        
        //Verifica a existência desse tipo de hierarquia
        if(empty($objTipohierarqorg)) {
            //Não existe este tipo de hierarquia, será criado:
            return $this->criaTipoHierarquiaLegalGeren($nrorg, $nrorgPadrao, $nrtipoestrutlegal, $nrtipoestrutgeren);
        } else {
            //O tipo de hierarquia já existe
            return array('nrtipohierarqorg' => $objTipohierarqorg->getNrtipohierarqorg(),
                         'nmtipohierarqorg' => $objTipohierarqorg->getNmtipohierarqorg());
        }
    }
    
    /**
     * Cria um tipo de hierarquia para estrutura legal/gerencial
     */
    private function criaTipoHierarquiaLegalGeren($nrorg, $nrorgPadrao, $nrtipoestrutlegal, $nrtipoestrutgeren) {
        
        //Adiquirindo o nome do tipos de estruturas legal e gerencial
        $params = array('nrorg'           => array($nrorg, $nrorgPadrao),
                        'nrtipoestrutura' => $nrtipoestrutlegal);
        $objTipoestruturaLegal = $this->entityManager->getRepository(Repositories::TIPOESTRUTURA)->findOneBy($params);
        $params['nrtipoestrutura'] = $nrtipoestrutgeren;
        $objTipoestruturaGeren = $this->entityManager->getRepository(Repositories::TIPOESTRUTURA)->findOneBy($params);
        
        if(empty($objTipoestruturaLegal))
            throw new \Exception("O tipo de estrutura legal número $nrtipoestrutlegal não foi encontrado na tabela de tipos de estruturas.");
        if(empty($objTipoestruturaGeren))
            throw new \Exception("O tipo de estrutura gerencial número $nrtipoestrutgeren não foi encontrado na tabela de tipos de estruturas.");
            
        //Nome e número da nova hierarquia
        $nmtipohierarqorg = $objTipoestruturaLegal->getNmtipoestrutura() . ' / ' . $objTipoestruturaGeren->getNmtipoestrutura(); //O nome é not null, por isso nçao testo sua existência
        $nrtipohierarqorg = \HCM\Util\NovoCodigo::geraCodigo($nrorgPadrao, 'TIPOHIERARQORG', 1, 12, true);
        
        unset($params, $objTipoestruturaLegal, $objTipoestruturaGeren);
        
        //Criando um novo tipo de hierarquia
        $tipoHierarqOrg = Repositories::TIPOHIERARQORG;
        $obj = new $tipoHierarqOrg;
        $obj->setNrorg           ($nrorg);
        $obj->setDtinclusao      (DateUtil::getDataAtual());
        $obj->setCdoperinclusao  ($this->hcmParameters->getCdoperador());
        $obj->setNrorginclusao   ($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtultatu        (DateUtil::getDataAtual());
        $obj->setCdoperultatu    ($this->hcmParameters->getCdoperador());
        $obj->setNrorgultatu     ($this->hcmParameters->getNrorgTrabalho());
        $obj->setNrtipoestrutsup ($nrtipoestrutlegal);
        $obj->setNrtipoestrutrel ($nrtipoestrutgeren);
        $obj->setNmtipohierarqorg($nmtipohierarqorg);
        $obj->setNrtipohierarqorg($nrtipohierarqorg);
        
        $this->entityManager->persist($obj);
        $this->entityManager->flush();
        
        return array('nrtipohierarqorg' => $nrtipohierarqorg,
                     'nmtipohierarqorg' => $nmtipohierarqorg); 
    
    }
    
    /**
     * Monta uma lista otimizada das estruturas superiores vs as suas relacionadas para se fazer a inserção na tabela de hierarquia
     */
    private function montaListaHierarquia($arrayVinculos) {
        
        //O primeiro índice é a estrutura superior (legal), o valor é um array com as estruturas relacionadas (gerencial)
        $HIERARQUIAS = array();
        $msg = array();
        
        foreach($arrayVinculos as $vinculo) {
            $nrlegal = $vinculo['NRESTRUTLEGAL'];
            $nrgeren = $vinculo['NRESTRUTGEREN'];
            
            //Apenas os vínculos com estruturas cadastradas
            if(is_null($nrlegal)) $msg[] = "Vínculo ".$vinculo['NRVINCULOM']." - ".$vinculo['NMVINCULOM']." sem estrutura legal.";
            if(is_null($nrgeren)) $msg[] = "Vínculo ".$vinculo['NRVINCULOM']." - ".$vinculo['NMVINCULOM']." sem estrutura gerencial.";
            if(is_null($nrlegal) || is_null($nrgeren))
                continue; //Aborta a iserção abaixo
            
            //verifica se já existe a superior
            if(isset($HIERARQUIAS[$nrlegal])) {
                //Estrutura superior já existente, verificando se sua relacionada já existe na lista
                if(!in_array($nrgeren, $HIERARQUIAS[$nrlegal])) {
                    $HIERARQUIAS[$nrlegal][] = $nrgeren;
                }
            } else {
                //Estrutura superior nova, logicamnete sua relacionada também é nova.
                $HIERARQUIAS[$nrlegal][] = $nrgeren;
            }
        }
        
        return array('HIERARQUIAS' => $HIERARQUIAS, 'semEstrutura' => $msg);
    }
    
    /**
     * Insere as hierarquias
     */
    private function insereHierarquias($nrorg, $mesCompetencia, $nrtipohierarqorg, $nmtipohierarqorg, $hierarquias) {

        $nmtipohierarqorg = substr($nmtipohierarqorg . ' - Cad. Automático', 0, 50); //Limite do nome da hierarquia
        $params = array('nrorg'            => $nrorg,
                        'nrtipohierarqorg' => $nrtipohierarqorg,
                        //'dtinivigencia'    => DateUtil::getDataDeString($mesCompetencia, DateUtil::FORMATO_BRASILEIRO, true),
                        'idativo'          => 'S',
                        'nrestruturamsup'  => '',
                        'nrestruturamrel'  => '');
        $contador = array('inseridos' => 0, 'naoinseridos' => 0);
        
        foreach($hierarquias as $superior => $relacionadas) {
            //Adiquirindo as relacionadas
            foreach($relacionadas as $relacionada) {
                
                //Verifica se já existe a hierarquia em questão
                $params['nrestruturamsup'] = $superior;
                $params['nrestruturamrel'] = $relacionada;
                $objHierarquiaorg = $this->entityManager->getRepository(Repositories::HIERARQUIAORG)->findOneBy($params);
                
                if(empty($objHierarquiaorg)) {
                    //Inserindo uma nova hierarquia
                    try{
                        //$this->entityManager->beginTransaction();
                        $hierarquiaorg = Repositories::HIERARQUIAORG;
                        $obj = new $hierarquiaorg;
                        
                        $obj->setDtinclusao      (DateUtil::getDataAtual());
                        $obj->setDtultatu        (DateUtil::getDataAtual());
                        $obj->setDtinivigencia   (DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
                        $obj->setNrorg           ($nrorg);
                        $obj->setCdoperinclusao  ($this->hcmParameters->getCdoperador());
                        $obj->setNrorginclusao   ($this->hcmParameters->getNrorgTrabalho());
                        $obj->setCdoperultatu    ($this->hcmParameters->getCdoperador());
                        $obj->setNrorgultatu     ($this->hcmParameters->getNrorgTrabalho());
                        $obj->setNrtipohierarqorg($nrtipohierarqorg);
                        $obj->setIdativo         ('S');
                        $obj->setNmhierarquiaOrg ($nmtipohierarqorg);
                        $obj->setNrestruturamsup ($superior);
                        $obj->setNrestruturamrel ($relacionada);
                        $obj->setNrhierarquiaorg(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'HIERARQUIAORG', 1, 22, true));
                        //$this->entityManager->commit();
                        $this->entityManager->persist($obj);
                        $this->entityManager->flush();
                        
                        $contador['inseridos']++;
                    } catch (\Exception $e) {
                        $this->entityManager->rollback();
                        throw $e;
                    }
                } else {
                    $contador['naoinseridos']++;
                }
            }
        }
        
        return $contador;
    }
    
    
    
    public function verificaTpEstrutura($nrorg, $nrorgpadrao){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        
        $paramOrg = $this->entityManager->getRepository(Repositories::GPE_PARAMORG)->findOneBy(array('nrorg' => $nrorg));
        $nrtpestrutlegal = $paramOrg->getNrtpestrutlegal();
        $nrtpestrutgeren = $paramOrg->getNrtpestrutgeren();
        if($paramOrg != null){    
            $tipohierarq = $this->entityManager->getRepository(Repositories::TIPOHIERARQORG)->findOneBy(array('nrorg' => array($nrorg,$nrorgpadrao),
            'nrtipoestrutsup' => $nrtpestrutlegal,
            'nrtipoestrutrel' => $nrtpestrutgeren
                ));
            $nrtipohierarqorg = $tipohierarq->getNrtipohierarqorg();
            return $nrtipohierarqorg; // retorna numero tipo de hierarquia
            
        }else{
            $objAuxSup = $this->entityManager->getRepository(Repositories::TIPOESTRUTURA)->findOneBy(array('nrorg' => array($nrorg, $nrorgpadrao),
            'nrtipoestrutura' => $nrtpestrutlegal));
        
            $objAuxRel = $this->entityManager->getRepository(Repositories::TIPOESTRUTURA)->findOneBy(array('nrorg' => array($nrorg, $nrorgpadrao),
            'nrtipoestrutura' => $nrtpestrutgeren));
        
            $nmtipoestrutura = $objAuxSup->getNmtipoestrutura() . ' / ' . $objAuxRel->getNmtipoestrutura();
            
        }
    }
    
    public function retornaHierarquiaGridExistente($dataset){
        $Message = '';
        
        $itemHierarquia = $this->hierarquiaGridRepository->retornaHierarquiaGridExistente($dataset['NRTIPOHIERARQORG'], $dataset['NRESTRUTURAMREL'], $dataset['NRORG']);
        
        if(array_key_exists(0, $itemHierarquia)) $itemHierarquia = $itemHierarquia[0];
        
        if($itemHierarquia){
            if($itemHierarquia['DTFIMVIGENCIA'] == null || DateUtil::getDataDeString($dataset['DTINIVIGENCIA']) < DateUtil::getDataDeString(substr($itemHierarquia['DTFIMVIGENCIA'], 0, 10)) 
            || substr($dataset['DTINIVIGENCIA'], 0, 10) == substr($itemHierarquia['DTFIMVIGENCIA'], 0, 10)){
                return true;
            }else{
                return false;
            }
        }else{
            return false;
        }
    }
    
    public function getHierarquiaGridTree($nrestruturamrel, $nrestruturamsup, $nrtipohierarqorg, $nrtipoestrutsup, $conditions, $buttonFloatingSearch){

        $where_placeholder = "1=1";

        if (count($conditions) > 0) $where_placeholder = implode(" AND ",$conditions);
        
        return $this->hierarquiaGridRepository->getHierarquiaGridTree($nrestruturamrel, $nrestruturamsup, $nrtipohierarqorg, $nrtipoestrutsup, $where_placeholder, $buttonFloatingSearch);

    }    
    
    public function getHierarquiaGrid($nrestruturamrel, $nrestruturamsup, $conditions, $buttonFloatingSearch){

        $where_placeholder = "1=1";

        if (count($conditions) > 0) $where_placeholder = implode(" AND ",$conditions);
        
        return $this->hierarquiaGridRepository->getHierarquiaGrid($nrestruturamrel, $nrestruturamsup, $where_placeholder, $buttonFloatingSearch);

    }

    public function setInformacoesParaValidacao(\Zeedhi\Framework\DTO\Row $row): void
    {
        $listaEstruturaSuperior = $this->hierarquiaGridRepository->getHierarquiaGrid($row["NRESTRUTURAMREL"], null, "1=1", null);

        for ($i = 0; $i < count($listaEstruturaSuperior); $i++) {
            $listaEstruturaSuperior[$i]["DTINIVIGENCIA"] = substr($listaEstruturaSuperior[$i]["DTINIVIGENCIA"], 0, 10);
        }

        $nrTipoEstruturaTomador = $this->entityManager->getRepository(Repositories::GPE_PARAMORG)->findOneBy(array('nrorg' => $row["NRORG"]))->getNrtpestrutomador();

        $this->informacoesParaValidacao = [
            "NRHIERARQUIAORG" => $row["NRHIERARQUIAORG"],
            "NRTIPOHIERARQORG" => $row["NRTIPOHIERARQORG"],
            "DTINIVIGENCIA" => substr($row["DTINIVIGENCIA"], 0, 10),
            "LISTA_ESTRUTURASUPERIOR" => $listaEstruturaSuperior,
            "NRTIPOESTRUTURATOMADOR" => $nrTipoEstruturaTomador
        ];
    }

    public function isHierarquiaGridInvalida(): bool
    {
        foreach ($this->informacoesParaValidacao["LISTA_ESTRUTURASUPERIOR"] as $estruturaSuperior) {

            /* Garante nao considerar o mesmo registro caso ele ja exista */
            if ($estruturaSuperior["NRHIERARQUIAORG"] == $this->informacoesParaValidacao["NRHIERARQUIAORG"]) {
                continue;
            }

            $tipoEstruturaIgualTipoEstruturaTomador = (
                ($estruturaSuperior["NRTIPOESTRUTURA"] == $this->informacoesParaValidacao["NRTIPOESTRUTURATOMADOR"])
                    ||
                ($estruturaSuperior["NRTIPOESTRUTURA"] == self::TIPO_ESTRUT_TOMADOR && $this->informacoesParaValidacao["NRTIPOESTRUTURATOMADOR"] == null)
            );

            $isInvalid = (
                ($tipoEstruturaIgualTipoEstruturaTomador || $estruturaSuperior["NRTIPOESTRUTURA"] == null)
                    &&
                ($estruturaSuperior["DTINIVIGENCIA"] == $this->informacoesParaValidacao["DTINIVIGENCIA"])
                    &&
                ($estruturaSuperior["NRTIPOHIERARQORG"] == $this->informacoesParaValidacao["NRTIPOHIERARQORG"])
            );

            if ($isInvalid) {
                return true;
            }
        }

        return false;
    }
    
        public function verificaEstruturaTomadorExistente($dataset) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrTipoTomador = $this->hcmParameters->getTipoEstruturaTomadorOrg() ? $this->hcmParameters->getTipoEstruturaTomadorOrg() : 7;
            $nrHierarquiaGrid = !empty($dataset['NRHIERARQUIAORG']) ? $dataset['NRHIERARQUIAORG'] : null;
            
            $estruturaTomador = $this->hierarquiaGridRepository->retornaGridExistenteValido($nrTipoTomador, $dataset['NRESTRUTURAMREL'], $nrorg, $nrHierarquiaGrid);
                
            if($estruturaTomador){ 
                return true;
            } else { 
                return false; 
            }
        } catch (\Exception $e) {            
            throw $e;
        }
    }

}