<?php

namespace HCM\PTO\Logic\AFD;

/**
 * Registro preparado para ser utilizado com queries
 * 
 * 
 */

abstract class RegistroPorConsulta extends Registro {
    /** @var string[] */
    private $itensUltimaGeracao;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Parametros $parametros, ?ValidadorRegistro $validadorRegistro = null) {
        $this->entityManager = $entityManager;
        parent::__construct($entityManager, $parametros, $validadorRegistro);
    }
    
    /**
     * Retorna a query a ser executada para a montagem dos itens do registro
     * @return string
     */
    public abstract function obtemConsultaRegistro(): string;
    
    /**
     * Retorna os parametros da query a ser executada para a montagem dos itens do registro
     * @return AssociativeArray
     */
    public abstract function obtemParametrosConsultaRegistro(): array;
    
    /**
     * Retorna os itens do registro atraves da execucao da query do registro
     * @retrun string[]
     */
    public function itens(): array {
        $itens = $this->entityManager->getConnection()->fetchAll(
            $this->obtemConsultaRegistro(),
            $this->obtemParametrosConsultaRegistro()
        );
        $this->itensUltimaGeracao = $itens;
        return self::itensConsultaToItens($itens) ?? [];
    }
    
    /**
     * Retorna os itens da ultima geracao
     * @retrun string[]
     */
    public function itensUltimaGeracao(): array {
        return $this->itensUltimaGeracao;
    }
    
    /**
     * Retorna a quantidade de itens da ultima geracao
     * @retrun int
     */
    public function quantidadeItensUltimaGeracao(): int {
        return count($this->itensUltimaGeracao);
    }
    
    /**
     * Transforma o array associativo retornado da execucao da query em um array convencional
     * @param string $itensConsulta
     * @retrun string[]
     */
    private static function itensConsultaToItens(array $itensConsulta): array {
        return array_map(
            function ($item) {
                return array_values($item)[0];
            },
            $itensConsulta
        );
    }
}