<?php

namespace HCM\PTO\Repository;

use Doctrine\ORM\EntityManager;

class OcorrenciaFrequenciaRepository {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    /**
     *  Rotina que aplica um script SQL responsavel por cadastrar ocorrencias do tipo "Abonar falta" para determinados vinculos em dias cadastrados no calendario da estrutura legal.
     * Por definicao, apenas os feriados apos a competencia informada serao cadastradas, com o objetivo de nao afetar periodos anteriores.
     * 
     *  @param  $nrOrg
     *  @param  $dtMesCompetenc
     *  @param  $nrVinculom
     *  @param  $nrEstrutLegal
     *  @param  $nrCalendario
     * 
     *  @return bool
     * 
     */
    public function atualizaFeriado(
        $nrOrg,
        $dtMesCompetenc,
        $nrVinculom         = null,
        $nrEstrutLegal      = null,
        $nrCalendario       = null
    ) {
        
        $params = [
            'P_NRORG'           => $nrOrg,
            'P_DTMESCOMPETENC'  => $dtMesCompetenc,
            'P_NRVINCULOM'      => $nrVinculom,
            'P_NRESTRUTLEGAL'   => $nrEstrutLegal,
            'P_NRCALENDARIO'    => $nrCalendario,
        ];
        
        $query = $this->entityManager->getConnection()->prepare("
            DECLARE
                V_NRORG                         NUMBER  := :P_NRORG;
                V_DTMESCOMPETENC                DATE    := :P_DTMESCOMPETENC;
                V_NRTIPOPROGOCORRENCIA_ABONO    NUMBER  := 5;
                V_NRVINCULOM                    NUMBER  := :P_NRVINCULOM;
                V_NRESTRUTLEGAL                 NUMBER  := :P_NRESTRUTLEGAL;
                V_NRCALENDARIO                  NUMBER  := :P_NRCALENDARIO;
            BEGIN
               FOR C_PRINC IN
                  (
                    SELECT
                        VM.NRVINCULOM,
                        VM.NMVINCULOM,
                        VH.DTMESCOMPETENC AS DTMESCOMPETENC_VINC,
                        VH.NRESTRUTLEGAL,
                        VH.NRESTRUTGEREN,
                        ESTRUT_CALEND.NRESTRUTURAM,
                        ESTRUT_CALEND.NMESTRUTURAM,
                        FER.DTCALENDARIO,
                        FER.DSOBSCALEND
                    FROM GPE_VINCULOM VM
                    JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                        SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH WHERE NRORG = VH.NRORG AND NRVINCULOM = VH.NRVINCULOM AND DTMESCOMPETENC <= V_DTMESCOMPETENC
                    )
                    JOIN ESTRUTURAM ESTRUT_CALEND ON ESTRUT_CALEND.NRORG = VM.NRORG
                        AND ESTRUT_CALEND.NRESTRUTURAM = (
                            SELECT
                                MOV.NRESTRUTURAM
                            FROM GPE_MOVIMENTACAO MOV
                            JOIN GPE_PARAMORG PO ON PO.NRORG = MOV.NRORG
                            WHERE MOV.NRORG = ESTRUT_CALEND.NRORG
                                AND MOV.NRVINCULOM = VM.NRVINCULOM
                                AND MOV.DTINIMOVIMENT <= LAST_DAY(V_DTMESCOMPETENC)
                                AND (MOV.DTFIMMOVIMENT IS NULL OR MOV.DTFIMMOVIMENT > LAST_DAY(V_DTMESCOMPETENC))
                                AND MOV.NRTIPOESTRUTURA IN (NVL(PO.NRTPESTRUTCALEND, 0), PO.NRTPESTRUTLEGAL)
                            ORDER BY DECODE(MOV.NRTIPOESTRUTURA, PO.NRTPESTRUTCALEND, 1, PO.NRTPESTRUTLEGAL, 2, 3), MOV.DTINIMOVIMENT DESC
                            FETCH FIRST ROW ONLY
                        )
                    JOIN ESTRUTURAH ESTRUT_CALENDH ON ESTRUT_CALENDH.NRORG = VM.NRORG AND ESTRUT_CALENDH.NRESTRUTURAM = ESTRUT_CALEND.NRESTRUTURAM
                        AND ESTRUT_CALENDH.DTMESCOMPETENC = (
                            SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH WHERE NRORG = ESTRUT_CALENDH.NRORG AND NRESTRUTURAM = ESTRUT_CALENDH.NRESTRUTURAM AND DTMESCOMPETENC <= V_DTMESCOMPETENC
                        )
                    JOIN OCORCALENDARIO FER ON FER.NRORG = VM.NRORG AND FER.NRCALENDARIO = ESTRUT_CALENDH.NRCALENDARIO
                        /* GARANTE QUE APENAS OS FERIADOS EXISTENTES APOS A ADMISSAO DO VINCULO SEJAM CONSIDERADOS */
                        AND FER.DTCALENDARIO >= VM.DTADMISSAOVINC
                        /* CONSIDERA APENAS FERIADOS A PARTIR DA COMPETENCIA ANTERIOR */
                        AND FER.DTCALENDARIO >= ADD_MONTHS(V_DTMESCOMPETENC, -1)
                    WHERE VM.NRORG = V_NRORG
                        AND (VM.NRVINCULOM      = V_NRVINCULOM      OR V_NRVINCULOM IS NULL)
                        AND (VH.NRESTRUTLEGAL   = V_NRESTRUTLEGAL   OR V_NRESTRUTLEGAL IS NULL)
                        AND (FER.NRCALENDARIO   = V_NRCALENDARIO    OR V_NRCALENDARIO IS NULL)
                        /* GARANTE QUE NAO EXISTE OCORRENCIA DE ABONO JA CADASTRADA */
                        AND NOT EXISTS (
                            SELECT 1
                            FROM PTO_PROGOCORRENCIA PO
                            WHERE PO.NRORG = VM.NRORG
                                AND PO.NRVINCULOM = VM.NRVINCULOM
                                AND TRUNC(PO.DTINICIOPROGOCOR) = TRUNC(PO.DTFIMPROGOCOR)
                                AND TRUNC(PO.DTINICIOPROGOCOR) = TRUNC(FER.DTCALENDARIO)
                                AND PO.NRTIPOPROGOCORRENCIA = 5
                                AND PO.IDTIPODIACALENDARIO = 'FERIADO'
                        )
                        /* VALIDACAO DE VINCULOS ATIVOS */
                        AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC > V_DTMESCOMPETENC)
                  )
               LOOP
                  INSERT INTO PTO_PROGOCORRENCIA (
                        NRORG, NRPROGOCORRENCIA, NRVINCULOM, DTINICIOPROGOCOR, DTFIMPROGOCOR, IDDIASSEMANA, IDTIPODIACALENDARIO, IDTIPOHORARIO,
                        NRMOTIVOOCORFREQ, NRHORDIAM, NRSITUFREQUENCIA, NRSITUFREQGATILHO, QTMINGATILHOPROGOCOR, QTMAXGATILHOPROGOCOR, VRCOEFICIENTEACRESCIMO, IDPROGOCORRENCIA,
                        QTMINIMAPROGOCOR, QTMAXIMAPROGOCOR, DSOBSERVACAO, IDATIVO, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO, NRORGULTATU,
                        DTULTATU, CDOPERULTATU, NRTIPOPROGOCORRENCIA, NRMARCACAOPONTO, NRINTERVALOHORARIO, IDPROGOCORCONFIRMADA, CDOPERVERIFICADO, DSOBSERVACAORECUSA,
                        HRPROJETARINTERVALONPROG, DSPATH, CDTABECDI, CDDIAGNOST, NRPESSOA
                    ) VALUES (
                        V_NRORG, (SELECT MAX(NRPROGOCORRENCIA) + 1 FROM PTO_PROGOCORRENCIA WHERE NRORG = V_NRORG), C_PRINC.NRVINCULOM, C_PRINC.DTCALENDARIO, C_PRINC.DTCALENDARIO, '1111111', 'FERIADO', 'N',
                        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                        NULL, NULL, C_PRINC.DSOBSCALEND, 'S', 1, SYSDATE, 1, NULL,
                        NULL, NULL, V_NRTIPOPROGOCORRENCIA_ABONO, NULL, NULL, 'S', NULL, NULL,
                        NULL, NULL, NULL, NULL, NULL
                    );
               END LOOP;
            END;"
        );

        return $query->execute($params);
    }
    
    /**
     *  Rotina que busca os calendario associados a um vinculo, estrutura legal ou a uma data especifica
     * 
     *  $nrOrg
     *  $dtCompetenc
     *  $dataCalendario     = null
     *  $nrVinculom         = null
     *  $nrEstrutLegal      = null
     * 
     *  @return Connection->fetchAssoc
     * 
     */
    public function getDiasCalendarioFeriado(
        $nrOrg,
        $dtCompetenc,
        $dataCalendario     = null,
        $nrVinculom         = null,
        $nrEstrutLegal      = null
    ) {
        $params = array(
           'P_NRORG'            => $nrOrg,
           'P_DTMESCOMPETENC'   => $dtCompetenc,
           'P_DTCALENDARIO'     => $dataCalendario,
           'P_NRVINCULOM'       => $nrVinculom,
           'P_NRESTRUTLEGAL'    => $nrEstrutLegal,
        );
        
        $sql = <<<SQL
            SELECT
                VM.NRVINCULOM,
                VM.NMVINCULOM,
                VM.DTADMISSAOVINC,
                EH_LEGAL.NRESTRUTURAM,
                EH_LEGAL.NMRAZSOCESTRUT,
                EH_LEGAL.NRCALENDARIO,
                OC.NROCORCALEND,
                OC.DTCALENDARIO,
                OC.DSOBSCALEND,
                OC.NRTPCALENDARIO,
                OC.DTINCLUSAO,
                OC.CDOPERINCLUSAO
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH WHERE NRORG = VH.NRORG AND NRVINCULOM = VH.NRVINCULOM AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
            )
            JOIN ESTRUTURAH EH_LEGAL ON EH_LEGAL.NRORG = VM.NRORG AND EH_LEGAL.NRESTRUTURAM = VH.NRESTRUTLEGAL AND EH_LEGAL.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH WHERE NRORG = EH_LEGAL.NRORG AND NRESTRUTURAM = EH_LEGAL.NRESTRUTURAM AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
            )
            JOIN OCORCALENDARIO OC ON OC.NRORG = VM.NRORG AND OC.NRCALENDARIO = EH_LEGAL.NRCALENDARIO
            WHERE VM.NRORG = :P_NRORG
                AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
                AND (OC.DTCALENDARIO = :P_DTCALENDARIO OR :P_DTCALENDARIO IS NULL)
                AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL OR :P_NRESTRUTLEGAL IS NULL)
SQL;
        
        return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
    }
    
}